@@ -1,7 +1,9 @@
 /*  This is the test file used to compile
-    testobjLE32PE.exe under MinGW on Windows 8.1 
+    testobjLE32PE.exe under MinGW on Windows 8.1
     These leading comments mean a recompile would
-    not exactly match line numbers in the DWARF. */
+    not exactly match line numbers in the DWARF.
+    This source file is hereby placed in the public domain.
+*/
 #include <stdio.h>
 
 struct something {
@@ -9,10 +11,10 @@ int a;
 unsigned b;
 };
 
-int 
+int
 buffle(struct something *v )
 {
-     return v->a + 42;
+    return v->a + 42;
 
 }
 
@@ -20,7 +22,7 @@ int main(int argc, char **argv)
 {
 
     int x = 12;
-     int y = 24;
+    int y = 24;
     struct something so;
 
     so.a = x;
