@@ -1,35 +1,7 @@
 /*
-  Copyright (c) 2019 David Anderson.  All rights reserved.
-
-  Redistribution and use in source and binary forms, with
-  or without modification, are permitted provided that the
-  following conditions are met:
-
-  * Redistributions of source code must retain the above
-    copyright notice, this list of conditions and the following
-    disclaimer.
-  * Redistributions in binary form must reproduce the above
-    copyright notice, this list of conditions and the following
-    disclaimer in the documentation and/or other materials
-    provided with the distribution.
-  * Neither the name of the example nor the
-    names of its contributors may be used to endorse or
-    promote products derived from this software without
-    specific prior written permission.
-
-  THIS SOFTWARE IS PROVIDED BY David Anderson ''AS IS'' AND ANY
-  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL David
-  Anderson BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
-  OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
+  Copyright (c) 2019 David Anderson.
+  This source code is placed in the Public Domain.
+  It may be copied or used in any way without restriction.
 */
 /*  findfuncbypc.c
     This is an example of code reading dwarf .debug_info.
@@ -75,7 +47,6 @@
 #ifdef HAVE_STRING_H
 #include <string.h> /* for strchr etc */
 #endif /* HAVE_STRING_H */
-#include <string.h>
 #ifdef HAVE_STDINT_H
 #include <stdint.h> /* For uintptr_t */
 #endif /* HAVE_STDINT_H */
@@ -188,7 +159,9 @@ printusage(void)
 static void target_data_destructor( struct target_data_s *td);
 
 static int
-startswithextractnum(const char *arg,const char *lookfor, Dwarf_Unsigned *numout)
+startswithextractnum(const char *arg,
+    const char                  *lookfor,
+    Dwarf_Unsigned              *numout)
 {
     const char *s = 0;
     unsigned prefixlen = strlen(lookfor);
@@ -294,10 +267,6 @@ resetsrcfiles(Dwarf_Debug dbg,struct srcfilesdata *sf)
 static void resetsubprog(Dwarf_Debug dbg,struct target_data_s *td)
 {
     td->td_subprog_haslowhighpc = FALSE;
-    if (td->td_subprog_name) {
-        dwarf_dealloc(dbg,td->td_subprog_name,DW_DLA_STRING);
-        td->td_subprog_name  = 0;
-    }
     if (td->td_subprog_die) {
         dwarf_dealloc(dbg,td->td_subprog_die,DW_DLA_DIE);
         td->td_subprog_die = 0;
@@ -1017,7 +986,8 @@ check_subprog_details(Dwarf_Debug dbg,
     int finalres = 0;
     int have_pc_range = FALSE;
 
-    res = getlowhighpc(dbg,td,die,level,&have_pc_range,&lowpc,&highpc,errp);
+    res = getlowhighpc(dbg,td,die,level,
+        &have_pc_range,&lowpc,&highpc,errp);
     if (res == DW_DLV_OK) {
         if (have_pc_range) {
             int res2 = DW_DLV_OK;
@@ -1042,10 +1012,6 @@ check_subprog_details(Dwarf_Debug dbg,
                     get_name_from_abstract_origin(dbg,is_info,
                         die, &name, errp);
                 }
-                if (td->td_subprog_name) {
-                    dwarf_dealloc(dbg,td->td_subprog_name,
-                        DW_DLA_STRING);
-                }
                 td->td_subprog_name = name;
                 name = 0;
                 /* Means this is an address match */
@@ -1300,7 +1266,7 @@ check_comp_dir(Dwarf_Debug dbg,Dwarf_Die die,
         &baseindex,
         &filescount,&endindex,errp);
 
-#endif
+#endif /* 0 */
 }
 
 static int
