@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2019, David Anderson
+Copyright (c) 2019-2020, David Anderson
 All rights reserved.
 
 Redistribution and use in source and binary forms, with
@@ -7,12 +7,13 @@ or without modification, are permitted provided that the
 following conditions are met:
 
     Redistributions of source code must retain the above
-    copyright notice, this list of conditions and the following
-    disclaimer.
+    copyright notice, this list of conditions and the
+    following disclaimer.
 
     Redistributions in binary form must reproduce the above
-    copyright notice, this list of conditions and the following
-    disclaimer in the documentation and/or other materials
+    copyright notice, this list of conditions and the
+    following disclaimer in the documentation and/or
+    other materials
     provided with the distribution.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
@@ -40,19 +41,18 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  */
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif /* HAVE_STRING_H */
 #ifdef HAVE_ELF_H
 #include <elf.h>
 #endif /* HAVE_ELF_H */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h> /* open(), off_t, size_t, ssize_t */
+#endif /* HAVE_SYS_TYPES_H */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h> /* getcwd */
 #endif /* HAVE_UNISTD_H */
-#if 0
-#include <sys/types.h> /* for open() */
-#include <sys/stat.h> /* for open() */
-#include <fcntl.h> /* for open() */
-#include <errno.h>
-#endif
 #include "dwarf_incl.h"
 #include "dwarf_alloc.h"
 #include "dwarf_error.h"
@@ -74,7 +74,6 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  */
 #define  UNUSEDARG
 #endif
 
-
 #if _WIN32
 #define NULL_DEVICE_NAME "NUL"
 #else
@@ -106,101 +105,30 @@ extract_buildid(Dwarf_Debug dbg,
     Dwarf_Error *error);
 
 struct joins_s {
-    char * js_fullpath;
     dwarfstring js_dirname;
-    dwarfstring js_basepath;
-    dwarfstring js_basename;
+    dwarfstring js_basenamesimple;
+    dwarfstring js_basesimpledebug;
+    dwarfstring js_binname;
     dwarfstring js_cwd;
     dwarfstring js_originalfullpath;
-    dwarfstring js_tmp;
+    dwarfstring js_linkstring;
     dwarfstring js_tmp2;
-    dwarfstring js_tmpdeb;
     dwarfstring js_tmp3;
     dwarfstring js_buildid;
     dwarfstring js_buildid_filename;
 };
 
-#if 0
-int
-_dwarf_check_string_valid(
-    void *areaptr,
-    void *strptr,
-    void *areaendptr,
-    int suggested_error,
-    int *errcode)
-{
-    Dwarf_Small *start = areaptr;
-    Dwarf_Small *p = strptr;
-    Dwarf_Small *end = areaendptr;
-    ptrdiff_t diff =  0;
-
-    if (p < start) {
-        diff = start - p;
-#ifdef  TESTING
-        printf("Error  string start  pointer error: loc %"
-            DW_PR_DSs
-            " bytes before available area \n",(Dwarf_Signed)diff);
-#endif  /* TESTING */
-        *errcode = suggested_error;
-        return DW_DLV_ERROR;
-    }
-    if (p >= end) {
-        diff = p - start;
-#ifdef  TESTING
-        printf("Error  string end  pointer error, not terminated %"
-            " before end of area. Length:  "
-            DW_PR_DSs  "\n",(Dwarf_Signed)diff);
-#endif  /* TESTING */
-        *errcode = suggested_error;
-        return DW_DLV_ERROR;
-    }
-    while (p < end) {
-        if (*p == 0) {
-            return DW_DLV_OK;
-        }
-        ++p;
-    }
-    diff =  p - start;
-#ifdef  TESTING
-    printf("Error string not terminated error:  not ended after %"
-        DW_PR_DSs " bytes (past end of available bytes)\n",
-        (Dwarf_Signed)diff);
-#endif  /* TESTING */
-    *errcode = DW_DLE_STRING_NOT_TERMINATED;
-    return DW_DLV_ERROR;
-}
-#endif
-
-
-#if 0
-static int
-does_file_exist(char *f)
-{
-    int fd = 0;
-
-    fd = open(f,O_RDONLY|O_BINARY);
-    if (fd < 0) {
-        return DW_DLV_NO_ENTRY;
-    }
-    /* Here we could derive the crc to validate the file. */
-    close(fd);
-    return DW_DLV_OK;
-}
-#endif
-
-
 static void
 construct_js(struct joins_s * js)
 {
     memset(js,0,sizeof(struct joins_s));
-    dwarfstring_constructor(&js->js_basename);
     dwarfstring_constructor(&js->js_dirname);
-    dwarfstring_constructor(&js->js_basepath);
+    dwarfstring_constructor(&js->js_basenamesimple);
+    dwarfstring_constructor(&js->js_basesimpledebug);
     dwarfstring_constructor(&js->js_cwd);
     dwarfstring_constructor(&js->js_originalfullpath);
-    dwarfstring_constructor(&js->js_tmp);
+    dwarfstring_constructor(&js->js_linkstring);
     dwarfstring_constructor(&js->js_tmp2);
-    dwarfstring_constructor(&js->js_tmpdeb);
     dwarfstring_constructor(&js->js_tmp3);
     dwarfstring_constructor(&js->js_buildid);
     dwarfstring_constructor(&js->js_buildid_filename);
@@ -209,13 +137,12 @@ static void
 destruct_js(struct joins_s * js)
 {
     dwarfstring_destructor(&js->js_dirname);
-    dwarfstring_destructor(&js->js_basepath);
-    dwarfstring_destructor(&js->js_basename);
+    dwarfstring_destructor(&js->js_basenamesimple);
+    dwarfstring_destructor(&js->js_basesimpledebug);
     dwarfstring_destructor(&js->js_cwd);
     dwarfstring_destructor(&js->js_originalfullpath);
-    dwarfstring_destructor(&js->js_tmp);
+    dwarfstring_destructor(&js->js_linkstring);
     dwarfstring_destructor(&js->js_tmp2);
-    dwarfstring_destructor(&js->js_tmpdeb);
     dwarfstring_destructor(&js->js_tmp3);
     dwarfstring_destructor(&js->js_buildid);
     dwarfstring_destructor(&js->js_buildid_filename);
@@ -223,19 +150,56 @@ destruct_js(struct joins_s * js)
 
 static char joinchar = '/';
 static char* joinstr = "/";
+#if defined (HAVE_WINDOWS_PATH)
+static char joincharw = '\\';
+/*  Large enough it is unlikely dwarfstring will ever
+    do a malloc here, paths usually < 100 characters. */
+static char winbuf[200];
 
+static void
+transformtoposix(dwarfstring * out, char *in)
+{
+    char *cp = in;
+
+    for( ; *cp  ; ++cp) {
+        if ( *cp == joincharw ) {
+            dwarfstring_append_length(out,joinstr,1);
+        } else {
+            dwarfstring_append_length(out,cp,1);
+        }
+    }
+}
+
+#endif /* HAVE_WINDOWS_PATH */
+
+/*  ASSERT: the target chars already have the transformtoposix()
+    done.
+*/
 int
 _dwarf_pathjoinl(dwarfstring *target,dwarfstring * input)
 {
     char *inputs = dwarfstring_string(input);
     char *targ = dwarfstring_string(target);
     size_t targlen = 0;
+#if defined (HAVE_WINDOWS_PATH)
+    dwarfstring winput;
 
-    if (!dwarfstring_strlen(target)) {
-        dwarfstring_append(target,dwarfstring_string(input));
+    dwarfstring_constructor_static(&winput,winbuf,sizeof(winbuf));
+    transformtoposix(&winput,inputs);
+    targlen = dwarfstring_strlen(target);
+    inputs = dwarfstring_string(&winput);
+    if (!targlen) {
+        dwarfstring_append(target,inputs);
         return DW_DLV_OK;
     }
+#else /* !HAVE_WINDOWS_PATH */
     targlen = dwarfstring_strlen(target);
+#endif /* HAVE_WINDOWS_PATH */
+
+    if (!targlen) {
+        dwarfstring_append(target,inputs);
+        return DW_DLV_OK;
+    }
     targ = dwarfstring_string(target);
     if (targ[targlen-1] != joinchar) {
         if (*inputs != joinchar) {
@@ -251,6 +215,9 @@ _dwarf_pathjoinl(dwarfstring *target,dwarfstring * input)
             dwarfstring_append(target,inputs+1);
         }
     }
+#if defined (HAVE_WINDOWS_PATH)
+    dwarfstring_destructor(&winput);
+#endif /* HAVE_WINDOWS_PATH */
     return DW_DLV_OK;
 }
 /*  ASSERT: the last character in s is not a /  */
@@ -267,10 +234,15 @@ mydirlen(char *s)
         }
     }
     if (lastjoinchar) {
-        /* we know diff is postive in all cases */
-        ptrdiff_t diff =  lastjoinchar - s;
+        /*  ptrdiff_t is generated but not named */
+        Dwarf_Unsigned sizetoendjoin =
+            (lastjoinchar >= s)?(lastjoinchar-s):0xffffffff;
         /* count the last join as mydirlen. */
-        return (size_t)(diff+1);
+        if (sizetoendjoin == 0xffffffff) {
+            /* impossible. */
+            return 0;
+        }
+        return sizetoendjoin;
     }
     return 0;
 }
@@ -335,6 +307,19 @@ dwarfstring_list_destructor(struct dwarfstring_list_s *l)
     }
 }
 
+static void
+prepare_linked_name(dwarfstring *out,
+    dwarfstring *dirname,
+    dwarfstring *extradir,
+    dwarfstring *linkname)
+{
+    dwarfstring_append(out, dwarfstring_string(dirname));
+    if (extradir) {
+        _dwarf_pathjoinl(out,extradir);
+    }
+    _dwarf_pathjoinl(out,linkname);
+}
+
 static void
 build_buildid_filename(dwarfstring *target,
     unsigned buildid_length,
@@ -343,19 +328,19 @@ build_buildid_filename(dwarfstring *target,
     dwarfstring tmp;
     unsigned bu = 0;
     unsigned char *cp  = 0;
-    char lbuf[10];
 
     dwarfstring_constructor(&tmp);
+    dwarfstring_append(&tmp,".build-id");
+    dwarfstring_append(&tmp,joinstr);
     cp = buildid;
     for (bu = 0; bu < buildid_length; ++bu ,++cp) {
-        sprintf(lbuf,"%02x",*cp);
-        dwarfstring_append(&tmp,lbuf);
+        dwarfstring_append_printf_u(&tmp, "%02x",*cp);
         if (bu == 0) {
             dwarfstring_append(&tmp,"/");
         }
     }
-    dwarfstring_append(&tmp,".debug");
     _dwarf_pathjoinl(target,&tmp);
+    dwarfstring_append(target,".debug");
     dwarfstring_destructor(&tmp);
     return;
 }
@@ -374,19 +359,44 @@ dump_bytes(const char *msg,unsigned char * start, unsigned len)
 }
 #endif
 
+/*  If the compiler or the builder of the object file
+    being read has made a mistake it is possible
+    for a constructed name to match the original
+    file path, yet we do not want to add that to
+    the search path. */
+static int
+duplicatedpath(dwarfstring* l,
+    dwarfstring*r)
+{
+    if (strcmp(dwarfstring_string(l),
+        dwarfstring_string(r))) {
+        return FALSE;
+    }
+    /*  Oops. somebody made a mistake */
+    return TRUE;
+}
+
+
 
 /*  New September 2019.  Access to the GNU section named
     .gnu_debuglink
     See
-    https://sourceware.org/gdb/onlinedocs/gdb/Separate-Debug-Files.html
-
+    https://sourceware.org/gdb/onlinedocs/
+        gdb/Separate-Debug-Files.html
+    The default global path was set
+    to /usr/lib/debug
+    by set_global_paths_init()
+    called at libdwarf object init time.
+    So there is always the default
+    global path present.
 */
-int _dwarf_construct_linkedto_path(
+int
+_dwarf_construct_linkedto_path(
    char         **global_prefixes_in,
    unsigned       length_global_prefixes_in,
    char          *pathname_in,
    char          *link_string_in, /* from debug link */
-   dwarfstring * link_string_fullpath_out,
+   dwarfstring   *link_string_fullpath_out,
    UNUSEDARG unsigned char *crc_in, /* from debug_link, 4 bytes */
    unsigned char *buildid, /* from gnu buildid */
    unsigned       buildid_length, /* from gnu buildid */
@@ -404,23 +414,16 @@ int _dwarf_construct_linkedto_path(
 
     dwarfstring_list_constructor(&base_dwlist);
     construct_js(&joind);
-    build_buildid_filename(&joind.js_buildid_filename,
-        buildid_length, buildid);
     dirnamelen = mydirlen(depath);
     if (dirnamelen) {
-        dwarfstring_append_length(&joind.js_dirname,
+        /*  Original dirname, before cwd (if needed) */
+        dwarfstring_append_length(&joind.js_tmp2,
             depath,dirnamelen);
     }
-    dwarfstring_append(&joind.js_basepath,depath+dirnamelen);
-    dwarfstring_append(&joind.js_basename,link_string_in);
     if (depath[0] != joinchar) {
+        /*  Meaning a/b or b, not /a/b or /b ,
+            so we apply cwd */
         char  buffer[2000];
-#ifdef TESTING
-        buffer[0] = 0;
-        /*  For testing lets use a fake (consistent)
-            base dir.  */
-        strcpy(buffer,"/fake/dir/path");
-#else
         unsigned buflen= sizeof(buffer);
         char *wdret = 0;
 
@@ -434,56 +437,86 @@ int _dwarf_construct_linkedto_path(
             *errcode = DW_DLE_ALLOC_FAIL;
             return DW_DLV_ERROR;
         }
-#endif /* TESTING */
         dwarfstring_append(&joind.js_cwd,buffer);
+        dwarfstring_append(&joind.js_dirname,buffer);
+        _dwarf_pathjoinl(&joind.js_dirname,&joind.js_tmp2);
         buffer[0] = 0;
+    }  else {/* else leave js_cwd empty */
+        dwarfstring_append(&joind.js_dirname,
+            dwarfstring_string(&joind.js_tmp2));
     }
 
-    {
-        /*  Builds the full path to the original
-            executable, but absent executable name. */
-        dwarfstring_append(&joind.js_originalfullpath,
-            dwarfstring_string(&joind.js_cwd));
-        _dwarf_pathjoinl(&joind.js_originalfullpath,
-            &joind.js_dirname);
-        _dwarf_pathjoinl(&joind.js_originalfullpath,
-            &joind.js_basepath);
-#ifdef TESTING
-        printf("originalfullpath    : %s\n",
-            dwarfstring_string(&joind.js_originalfullpath));
-#endif
-    }
-    {
-        /*  There is perhaps a directory prefix in the
-            incoming pathname.
-            So we add that to js_cwd. */
-        res = _dwarf_pathjoinl(&joind.js_cwd,
-            &joind.js_dirname);
-        /* This is used in a couple search paths. */
-    }
+
+    /* Creating a real basename. No slashes. */
+    dwarfstring_append(&joind.js_basenamesimple,
+        depath+dirnamelen);
+    dwarfstring_append(&joind.js_basesimpledebug,
+        dwarfstring_string(&joind.js_basenamesimple));
+    dwarfstring_append(&joind.js_basesimpledebug,".debug");
+
+    build_buildid_filename(&joind.js_buildid_filename,
+        buildid_length, buildid);
+    dwarfstring_reset(&joind.js_tmp2);
+
+    /*  Now js_dirname / js_basenamesimple
+        are reflecting a full path  */
+
+    /*  saves the full path to the original
+        executable */
+    dwarfstring_append(&joind.js_originalfullpath,
+        dwarfstring_string(&joind.js_dirname));
+    _dwarf_pathjoinl(&joind.js_originalfullpath,
+        &joind.js_basenamesimple);
+
+    /*  js_originalfullpath is a true full path
+        to the object we started with as input. */
+    /*  First, do the build-id method if it applies
+        at all */
     for (global_prefix_number = 0;
         buildid_length &&
         (global_prefix_number < length_global_prefixes_in);
         ++global_prefix_number) {
         char * prefix = 0;
+        struct dwarfstring_list_s *now_last = 0;
 
         prefix = global_prefixes_in[global_prefix_number];
         dwarfstring_reset(&joind.js_buildid);
         dwarfstring_append(&joind.js_buildid,prefix);
         _dwarf_pathjoinl(&joind.js_buildid,
             &joind.js_buildid_filename);
-        if (!strcmp(dwarfstring_string(&joind.js_originalfullpath),
-            dwarfstring_string(&joind.js_buildid))) {
-#ifdef TESTING
-            printf("duplicated output string %s\n",
-                dwarfstring_string(&joind.js_buildid));
-#endif /* TESTING */
-            /* duplicated name. spurious match. */
-        } else {
-            struct dwarfstring_list_s *now_last = 0;
+        res = dwarfstring_list_add_new(
+            &base_dwlist,
+            last_entry,&joind.js_buildid,
+            &now_last,errcode);
+        if(res != DW_DLV_OK) {
+            dwarfstring_list_destructor(&base_dwlist);
+            destruct_js(&joind);
+            return res;
+        }
+            last_entry = now_last;
+    } /* end loop on global prefix numbers */
+    /* end of the build-id name-finding. */
+
+    /*  Now for the debug link method of finding debug
+        files. */
+    if (link_string_in) {
+        /* First try dir of executable */
+        struct dwarfstring_list_s *now_last = 0;
+
+        dwarfstring_append(&joind.js_linkstring,
+            link_string_in);
+        dwarfstring_reset(&joind.js_tmp3);
+        dwarfstring_reset(&joind.js_tmp2);
+        prepare_linked_name(&joind.js_tmp3,
+            &joind.js_dirname,
+            0,&joind.js_linkstring);
+        dwarfstring_append(link_string_fullpath_out,
+            dwarfstring_string(&joind.js_tmp3));
+        if (!duplicatedpath(&joind.js_originalfullpath,
+            &joind.js_tmp3)) {
             res = dwarfstring_list_add_new(
                 &base_dwlist,
-                last_entry,&joind.js_buildid,
+                last_entry,&joind.js_tmp3,
                 &now_last,errcode);
             if(res != DW_DLV_OK) {
                 dwarfstring_list_destructor(&base_dwlist);
@@ -492,113 +525,60 @@ int _dwarf_construct_linkedto_path(
             }
             last_entry = now_last;
         }
-    }
-    if (link_string_in) {
-        /* js_cwd is a leading / directory name. */
-        {
-            dwarfstring_reset(&joind.js_tmp);
-            dwarfstring_append(&joind.js_tmp,
-                dwarfstring_string(&joind.js_cwd));
-            /* If we add basename do we find what we look for? */
-            res = _dwarf_pathjoinl(&joind.js_tmp,&joind.js_basename);
-            /* We return the original link as full path this way. */
-            dwarfstring_append(link_string_fullpath_out,
-                dwarfstring_string(&joind.js_tmp));
-            if (!strcmp(dwarfstring_string(&joind.js_originalfullpath),
-                dwarfstring_string(&joind.js_tmp))) {
-#ifdef TESTING
-                printf("duplicated output string %s\n",
-                    dwarfstring_string(&joind.js_tmp));
-#endif /* TESTING */
-                /* duplicated name. spurious match. */
-            } else if (res == DW_DLV_OK) {
-                struct dwarfstring_list_s *now_last = 0;
+        dwarfstring_reset(&joind.js_tmp2);
+        dwarfstring_reset(&joind.js_tmp3);
+        dwarfstring_append(&joind.js_tmp3,".debug");
+        prepare_linked_name(&joind.js_tmp2,
+            &joind.js_dirname,
+            &joind.js_tmp3,&joind.js_linkstring);
+        if (! duplicatedpath(&joind.js_originalfullpath,
+            &joind.js_tmp2)) {
+            res = dwarfstring_list_add_new(
+                &base_dwlist,
+                last_entry,&joind.js_tmp2,
+                &now_last,errcode);
+            if(res != DW_DLV_OK) {
+                dwarfstring_list_destructor(&base_dwlist);
+                destruct_js(&joind);
+                return res;
+            }
+            last_entry = now_last;
+        }
+        /*  Now look in the global locations. */
+        for (global_prefix_number = 0;
+            global_prefix_number < length_global_prefixes_in;
+            ++global_prefix_number) {
+            char * prefix =
+                global_prefixes_in[global_prefix_number];
+
+            dwarfstring_reset(&joind.js_tmp2);
+            dwarfstring_reset(&joind.js_tmp3);
+            dwarfstring_append(&joind.js_tmp2, prefix);
+            prepare_linked_name(&joind.js_tmp3,
+                &joind.js_tmp2,
+                &joind.js_dirname,
+                &joind.js_linkstring);
+            if (!duplicatedpath(&joind.js_originalfullpath,
+                &joind.js_tmp3)) {
                 res = dwarfstring_list_add_new(
                     &base_dwlist,
-                    last_entry,&joind.js_tmp,
+                    last_entry,&joind.js_tmp3,
                     &now_last,errcode);
                 if(res != DW_DLV_OK) {
                     dwarfstring_list_destructor(&base_dwlist);
-                    destruct_js(&joind);
+                        destruct_js(&joind);
                     return res;
                 }
                 last_entry = now_last;
             }
         }
-        {
-            dwarfstring_reset(&joind.js_tmp2);
-            dwarfstring_reset(&joind.js_tmpdeb);
-
-            dwarfstring_append(&joind.js_tmp2,
-                dwarfstring_string(&joind.js_cwd));
-            dwarfstring_append(&joind.js_tmpdeb,".debug");
-            res = _dwarf_pathjoinl(&joind.js_tmp2,&joind.js_tmpdeb);
-            if (res == DW_DLV_OK) {
-                res = _dwarf_pathjoinl(&joind.js_tmp2,
-                    &joind.js_basename);
-                /*  this the second search path
-                    after global directories
-                    search for nn/nnnnn....debug.   */
-                if (!strcmp(dwarfstring_string(
-                    &joind.js_originalfullpath),
-                    dwarfstring_string(&joind.js_tmp2))) {
-#ifdef TESTING
-                printf("duplicated output string %s\n",
-                    dwarfstring_string(&joind.js_tmp2));
-#endif /* TESTING */
-                    /* duplicated name. spurious match. */
-                } else if(res == DW_DLV_OK) {
-                    struct dwarfstring_list_s *now_last = 0;
-                    res = dwarfstring_list_add_new(
-                        &base_dwlist,
-                        last_entry,&joind.js_tmp2,
-                        &now_last,errcode);
-                    if(res != DW_DLV_OK) {
-                        dwarfstring_list_destructor(&base_dwlist);
-                        destruct_js(&joind);
-                        return res;
-                    }
-                    last_entry = now_last;
-                }
-            }
-        }
-        /*  Not found above, now look in the global locations. */
-        for (global_prefix_number = 0;
-            global_prefix_number < length_global_prefixes_in;
-            ++global_prefix_number) {
-            char * prefix = global_prefixes_in[global_prefix_number];
-
-            dwarfstring_reset(&joind.js_tmp3);
-            dwarfstring_append(&joind.js_tmp3, prefix);
-            res = _dwarf_pathjoinl(&joind.js_tmp3, &joind.js_cwd);
-            if (res == DW_DLV_OK) {
-                res = _dwarf_pathjoinl(&joind.js_tmp3,
-                    &joind.js_basename);
-                if (!strcmp(dwarfstring_string(
-                    &joind.js_originalfullpath),
-                    dwarfstring_string(&joind.js_tmp3))) {
-                    /* duplicated name. spurious match. */
-#ifdef TESTING
-                    printf("duplicated output string %s\n",
-                        dwarfstring_string(&joind.js_tmp3));
-#endif /* TESTING */
-                } else if (res == DW_DLV_OK) {
-                    struct dwarfstring_list_s *now_last = 0;
-                    res = dwarfstring_list_add_new(
-                        &base_dwlist,
-                        last_entry,&joind.js_tmp3,
-                        &now_last,errcode);
-                    if(res != DW_DLV_OK) {
-                        dwarfstring_list_destructor(&base_dwlist);
-                        destruct_js(&joind);
-                        return res;
-                    }
-                    last_entry = now_last;
-                }
-            }
-        }
     }
 
+    /*  Now malloc space for the pointer array
+        and the strings they point at so a simple
+        free by our caller will clean up
+        everything.  Copy the data from
+        base_dwlist to the new area. */
     {
         struct dwarfstring_list_s *cur = 0;
         char **resultfullstring = 0;
@@ -623,10 +603,6 @@ int _dwarf_construct_linkedto_path(
         resultfullstring = (char **)malloc(totalareasize);
         setstrindex = pointerarraysize;
         if(!resultfullstring) {
-#ifdef TESTING
-            printf("Malloc fail making final paths. Length %lu"
-                " bytes.\n",totalareasize);
-#endif /* TESTING */
             dwarfstring_list_destructor(&base_dwlist);
             destruct_js(&joind);
             *errcode = DW_DLE_ALLOC_FAIL;
@@ -737,12 +713,6 @@ extract_buildid(Dwarf_Debug dbg,
     secsize = pbuildid->dss_size;
     ptr = pbuildid->dss_data;
     if (secsize < sizeof(struct buildid_s)) {
-#ifdef  TESTING
-        printf("ERROR section .note.gnu.build-id too small: "
-            " section length: 0x%" DW_PR_DUx
-            " minimum struct size 0x%" DW_PR_DUx  "\n",
-            secsize,(Dwarf_Unsigned) sizeof(struct buildid_s));
-#endif /* TESTING */
         _dwarf_error(dbg,error,DW_DLE_CORRUPT_NOTE_GNU_DEBUGID);
         return DW_DLV_ERROR;
     }
@@ -785,11 +755,17 @@ extract_buildid(Dwarf_Debug dbg,
     return DW_DLV_OK;
 }
 
-/* */
-int dwarf_gnu_debuglink(Dwarf_Debug dbg,
-    char     **  debuglink_path_returned,
+/*  Caller frees space returned  by debuglink_fillpath_returned and
+    The following return pointers into the dbg itself
+    and are only valid while that dbg is open.
+    debuglink_path_returned.
+    crc_returned, buildid_owner_name_returned,
+    buildid_returned, */
+int
+dwarf_gnu_debuglink(Dwarf_Debug dbg,
+    char     **  debuglink_path_returned, /* do not free*/
     unsigned char **  crc_returned,
-    char     **  debuglink_fullpath_returned,
+    char     **  debuglink_fullpath_returned, /* caller frees */
     unsigned *   debuglink_fullpath_length_returned,
 
     unsigned *   buildid_type_returned ,
@@ -892,6 +868,7 @@ dwarf_add_debuglink_global_path(Dwarf_Debug dbg,
     const char **glpaths = 0;
     const char * path1 = 0;
 
+
     glpath_count_in = dbg->de_gnu_global_path_count;
     glpath_count_out = glpath_count_in+1;
     glpaths = (const char **)malloc(sizeof(char *)*
