@@ -1,26 +1,29 @@
 /*
-  Copyright (C) 2000-2005 Silicon Graphics, Inc. All Rights Reserved.
-  Portions Copyright (C) 2008-2018 David Anderson.  All Rights Reserved.
+Copyright (C) 2000-2005 Silicon Graphics, Inc. All Rights Reserved.
+Portions Copyright (C) 2008-2020 David Anderson.  All Rights Reserved.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of version 2.1 of the GNU Lesser General Public License
-  as published by the Free Software Foundation.
+  This program is free software; you can redistribute it
+  and/or modify it under the terms of version 2.1 of the
+  GNU Lesser General Public License as published by the Free
+  Software Foundation.
 
-  This program is distributed in the hope that it would be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  This program is distributed in the hope that it would be
+  useful, but WITHOUT ANY WARRANTY; without even the implied
+  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+  PURPOSE.
 
-  Further, this software is distributed without any warranty that it is
-  free of the rightful claim of any third person regarding infringement
-  or the like.  Any license provided herein, whether implied or
-  otherwise, applies only to this software file.  Patent licenses, if
-  any, provided herein do not apply to combinations of this program with
-  other software, or any other product whatsoever.
+  Further, this software is distributed without any warranty
+  that it is free of the rightful claim of any third person
+  regarding infringement or the like.  Any license provided
+  herein, whether implied or otherwise, applies only to this
+  software file.  Patent licenses, if any, provided herein
+  do not apply to combinations of this program with other
+  software, or any other product whatsoever.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this program; if not, write the Free Software
-  Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston MA 02110-1301,
-  USA.
+  You should have received a copy of the GNU Lesser General
+  Public License along with this program; if not, write the
+  Free Software Foundation, Inc., 51 Franklin Street - Fifth
+  Floor, Boston MA 02110-1301, USA.
 
 */
 #ifndef DWARF_ERRMSG_LIST_H
@@ -111,13 +114,13 @@ const char *_dwarf_errmsgs[] = {
         "section is allowed",
     "DW_DLE_DEBUG_STR_NULL (46) .debug_str section present but "
         "elf_getdata() failed or section is zero-length",
-    "DW_DLE_CU_LENGTH_ERROR (47)",
-    "DW_DLE_VERSION_STAMP_ERROR (48)",
-    "DW_DLE_ABBREV_OFFSET_ERROR (49)",
+    "DW_DLE_CU_LENGTH_ERROR (47) Corrupted DWARF or corrupted object",
+    "DW_DLE_VERSION_STAMP_ERROR (48) Corrupted DWARF or corrupted object",
+    "DW_DLE_ABBREV_OFFSET_ERROR (49) Corrupted DWARF or corrupted object",
     "DW_DLE_ADDRESS_SIZE_ERROR (50) size too large",
     "DW_DLE_DEBUG_INFO_PTR_NULL (51)",
     "DW_DLE_DIE_NULL (52)",
-    "DW_DLE_STRING_OFFSET_BAD (53)",
+    "DW_DLE_STRING_OFFSET_BAD (53) Corrupted DWARF or corrupted object",
     "DW_DLE_DEBUG_LINE_LENGTH_BAD (54)",
     "DW_DLE_LINE_PROLOG_LENGTH_BAD (55)",
     "DW_DLE_LINE_NUM_OPERANDS_BAD (56)",
@@ -126,7 +129,7 @@ const char *_dwarf_errmsgs[] = {
     "DW_DLE_DWARF_LINE_NULL (59)",
     "DW_DLE_INCL_DIR_NUM_BAD (60)",
     "DW_DLE_LINE_FILE_NUM_BAD (61)",
-    "DW_DLE_ALLOC_FAIL (62)",
+    "DW_DLE_ALLOC_FAIL (62) Out of memory or corrupted object",
     "DW_DLE_NO_CALLBACK_FUNC (63)",
     "DW_DLE_SECT_ALLOC (64)",
     "DW_DLE_FILE_ENTRY_ALLOC (65)",
@@ -479,7 +482,7 @@ const char *_dwarf_errmsgs[] = {
     "DW_DLE_STR_OFFSETS_ARRAY_SIZE(405) Not a multiple of entry size",
     "DW_DLE_STR_OFFSETS_VERSION_WRONG(406) Must be 5 ",
     "DW_DLE_STR_OFFSETS_ARRAY_INDEX_WRONG(407) Requested outside bound",
-    "DW_DLE_STR_OFFSETS_EXTRA_BYTES(408) Unused non-zero bytes end section",
+    "DW_DLE_STR_OFFSETS_EXTRA_BYTES(408) .debug_str_offsets section problem",
     "DW_DLE_DUP_ATTR_ON_DIE(409) Compiler error, object improper DWARF",
     "DW_DLE_SECTION_NAME_BIG(410) Caller provided insufficient room for section name",
     "DW_DLE_FILE_UNAVAILABLE(411). Unable find/read object file",
@@ -506,7 +509,7 @@ const char *_dwarf_errmsgs[] = {
     "DW_DLE_NO_ELF_SUPPORT(432) libdwarf was compiled without Elf object support.",
     "DW_DLE_NO_STREAM_RELOC_SUPPORT(433) no elf.h so cannot generate STREAM relocations",
     "DW_DLE_RETURN_EMPTY_PUBNAMES_ERROR(434) Flag value passed in not allowed.",
-    "DW_DLE_SECTION_SIZE_ERROR(435) Corrupt Elf. Section size not a multiple of section entry size",
+    "DW_DLE_SECTION_SIZE_ERROR(435) Corrupt Elf. Section size: greater than file size or not a multiple of section entry size",
     "DW_DLE_INTERNAL_NULL_POINTER(436) Internal libdwarf call:null pointer",
     "DW_DLE_SECTION_STRING_OFFSET_BAD(437) Corrupt Elf, an offset to section name is invalid",
     "DW_DLE_SECTION_INDEX_BAD(438) Corrupt Elf, a section index is incorrect",
@@ -537,7 +540,20 @@ const char *_dwarf_errmsgs[] = {
     "DW_DLE_PRO_INIT_EXTRAS_ERR(463) dwarf_producer_init() extras string has an unexpected space character",
     "DW_DLE_NULL_ARGS_DWARF_ADD_PATH(464) either Dwarf_Debug or file_path argument to dwarf_add_file_path is NULL.",
     "DW_DLE_DWARF_INIT_DBG_NULL(465) a dwarf_init*() call the return-dbg argument is null",
-
-
+    "DW_DLE_ELF_RELOC_SECTION_ERROR(466) A relocation section header link field is incorrect.",
+    "DW_DLE_USER_DECLARED_ERROR(467) library user created this.",
+    "DW_DLE_RNGLISTS_ERROR(468) Corrupt dwarf. Bad .debug_rnglists data.",
+    "DW_DLE_LOCLISTS_ERROR(469) Corrupt dwarf. Bad .debug_loclists data.",
+    "DW_DLE_SECTION_SIZE_OR_OFFSET_LARGE(470) corrupt section header.",
+    "DW_DLE_GDBINDEX_STRING_ERROR(471) .gdb_index section string error",
+    "DW_DLE_GNU_PUBNAMES_ERROR(472) A problem with .debug_gnu_pubnames",
+    "DW_DLE_GNU_PUBTYPES_ERROR(473) A problem with .debug_gnu_pubtypes",
+    "DW_DLE_DUPLICATE_GNU_DEBUG_PUBNAMES(474) Duplicated section .debug_gnu_pubnames",
+    "DW_DLE_DUPLICATE_GNU_DEBUG_PUBTYPES(475) Duplicated section .debug_gnu_pubtypes",
+    "DW_DLE_DEBUG_SUP_STRING_ERROR(476) The string in .debug_sup head runs off the end of the section. Corrupt data",
+    "DW_DLE_DEBUG_SUP_ERROR(477). .debug_sup data corruption",
+    "DW_DLE_LOCATION_ERROR(478). A location processing libdwarf error",
+    "DW_DLE_DEBUGLINK_PATH_SHORT(479) Buffer provided for GNU debuglink is too small",
+    "DW_DLE_SIGNATURE_MISMATCH(480) DWARF4 extension dwo_id and dwarf5signature present but they do not match!",
 };
 #endif /* DWARF_ERRMSG_LIST_H */
