@@ -71,6 +71,11 @@
 
 */
 
+struct Dwarf_Rnglists_Context_s;
+typedef struct Dwarf_Rnglists_Context_s *Dwarf_Rnglists_Context;
+struct Dwarf_Loclists_Context_s;
+typedef struct Dwarf_Loclists_Context_s *Dwarf_Loclists_Context;
+
 struct Dwarf_Die_s {
     Dwarf_Byte_Ptr di_debug_ptr;
     Dwarf_Abbrev_List di_abbrev_list;
@@ -96,6 +101,7 @@ struct Dwarf_Attribute_s {
         /*  If DW_FORM_implicit const, the value is here, not
             in the DIE. */
     Dwarf_Signed ar_implicit_const;
+    Dwarf_Debug  ar_dbg; /* dbg owning the attr */
 
     Dwarf_Die ar_die;/* Access to the DIE owning the attribute */
     Dwarf_Attribute ar_next;
@@ -105,14 +111,15 @@ struct Dwarf_Attribute_s {
 /*
     This structure provides the context for a compilation unit.
     Thus, it contains the Dwarf_Debug, cc_dbg, that this cu
-    belongs to.  It contains the information in the compilation
-    unit header, cc_length, cc_version_stamp, cc_abbrev_offset,
+    belongs to.  It contains the information
+    in the compilation unit header, cc_length,
+    cc_version_stamp, cc_abbrev_offset,
     and cc_address_size, in the .debug_info section for that cu.
     In addition, it contains the count, cc_count_cu, of the cu
     number of that cu in the list of cu's in the .debug_info.
     The count starts at 1, ie cc_count_cu is 1 for the first cu,
     2 for the second and so on.  This struct also contains a
-    pointer, cc_abbrev_table, to a list of pairs of abbrev code
+    pointer, to a list of pairs of abbrev code
     and a pointer to the start of that abbrev
     in the .debug_abbrev section.
 
@@ -166,6 +173,8 @@ struct Dwarf_CU_Context_s {
 
     /*  cc_debug_offset is the global offset in the section
         of the CU header of this CU.
+        The length of the CU is at offset
+        cc_debug_offset-cc_length_size-cc_extension_size;
         That is, it is a section global offset.
         May be debug_info or debug_types
         but those are distinct.
@@ -189,10 +198,12 @@ struct Dwarf_CU_Context_s {
             of some compilers to include dwo_id, but
             in a messier way(lacking DW_UT_*).
         If cc_unit_type == DW_UT_type ( DW_UT_split_type
-            was never part of DW5, never standard).
+            was never part of DW5, never standard,
+            though DW_UT_split_compile is DW5.).
             the signature is a type signature. */
 
     Dwarf_Half  cc_cu_die_tag;
+
     Dwarf_Sig8  cc_signature;
 
     /*  cc_type_signature_offset contains the
@@ -215,11 +226,12 @@ struct Dwarf_CU_Context_s {
 
     Dwarf_Bool cc_signature_present; /* Meaning type signature
         in TU header or, for CU header, signature in CU DIE. */
+    Dwarf_Bool cc_low_pc_present;
     Dwarf_Bool cc_addr_base_present;   /* Not TRUE in .dwo */
-    Dwarf_Bool cc_ranges_base_present; /* Not TRUE in .dwo */
-    Dwarf_Bool cc_str_offsets_base_present;
-    Dwarf_Bool cc_loclists_base_present;
+
     Dwarf_Bool cc_cu_die_has_children;
+    Dwarf_Bool cc_dwo_name_present;
+    Dwarf_Bool cc_at_strx_present;
 
     /*  Non zero if this context is a dwo section. Either
         dwo or dwp file. */
@@ -227,37 +239,82 @@ struct Dwarf_CU_Context_s {
 
     /*  cc_cu_die_offset_present is non-zero if
         cc_cu_die_global_sec_offset is meaningful.  */
-    Dwarf_Bool cc_cu_die_offset_present;
+    Dwarf_Bool     cc_cu_die_offset_present;
 
-    /*  FIXME from DW_AT_addr_base in CU DIE */
-    Dwarf_Unsigned cc_addr_base;     /* Zero in .dwo */
-    /*  FIXME from DW_AT_rnglists_base in CU DIE */
-    Dwarf_Unsigned cc_ranges_base;   /* Zero in .dwo */
-    /*  FIXME from DW_AT_str_offsets_base in CU DIE */
-    Dwarf_Unsigned cc_str_offsets_base;
-    char *     cc_dwo_name;
+    /* if present, is base address of CU */
+    Dwarf_Unsigned cc_low_pc;
+    /*  from DW_AT_addr_base in CU DIE, offset to .debug_addr table */
+    Dwarf_Unsigned cc_addr_base;  /* Zero in .dwo */
+
+    /*  DW_SECT_LINE */
+    Dwarf_Bool     cc_line_base_present;     /*DW5 */
+    Dwarf_Unsigned cc_line_base;             /*DW5 */
+    Dwarf_Unsigned cc_line_base_contr_size;  /*DW5 */
+
+    /*  From DW_AT_loclists_base or DW_SECT_LOCLISTS */
     Dwarf_Unsigned cc_loclists_base;
+    Dwarf_Unsigned cc_loclists_base_contr_size;
+    Dwarf_Bool     cc_loclists_base_present;
+    Dwarf_Bool     cc_loclists_header_length_present;
+
+    /*  .debug_str_offsets DW_SECT_STR_OFFSETS DW4 DW5 vs
+        DW_AT_str_offsets_base (table array off) */
+    Dwarf_Bool     cc_str_offsets_base_present;
+    Dwarf_Bool     cc_str_offsets_header_length_present;
+    Dwarf_Unsigned cc_str_offsets_header_offset; /* from cu/tu*/
+    Dwarf_Unsigned cc_str_offsets_contr_size;
+    Dwarf_Unsigned cc_str_offsets_base;
+    /*  to get from the start of a str_offsets table to the
+        offsets array entries.
+        See cc_str_offsets_header_length_present,
+        though not normally needed. If header_length
+        is zero all CUs in this DWP
+        use a DWARF4 extension simple offset array,
+        not a DWARF5 set of tables. */
+    Dwarf_Unsigned cc_str_offsets_header_length;
+    Dwarf_Unsigned cc_str_offsets_offset_size;
+
+    /*  DW_SECT_MACRO */
+    Dwarf_Unsigned cc_macro_base;    /*DW5 */
+    Dwarf_Unsigned cc_macro_base_contr_size;    /*DW5 */
+    Dwarf_Bool     cc_macro_base_present;
+    Dwarf_Bool     cc_macro_header_length_present;
+
+    /*  DW_SECT_RNGLISTS  */
+    Dwarf_Unsigned cc_rnglists_base;    /*DW5 */
+    Dwarf_Unsigned cc_rnglists_base_contr_size;    /*DW5 */
+    /*  DW_AT_GNU_ranges_base was a GNU extension that appeared
+        but was unused. See dwarf_die_deliv.c for details. */
+    Dwarf_Unsigned cc_ranges_base;
+    /*  DW_AT_GNU_ranges_base is a GNU extension, DW4  */
+    Dwarf_Bool     cc_ranges_base_present;
+    /* .debug_rnglists */
+    Dwarf_Bool     cc_rnglists_base_present; /* DW5 */
+    Dwarf_Bool     cc_rnglists_header_length_present;
+
+    char *         cc_dwo_name;
     /* === END DEBUG FISSION (Split Dwarf) data */
 
     /*  Global section offset to the bytes of the CU die for this CU.
         Set when the CU die is accessed by dwarf_siblingof_b(). */
     Dwarf_Unsigned cc_cu_die_global_sec_offset;
 
-    Dwarf_Byte_Ptr cc_last_abbrev_ptr;
-    Dwarf_Byte_Ptr cc_last_abbrev_endptr;
+
+    Dwarf_Byte_Ptr   cc_last_abbrev_ptr;
+    Dwarf_Byte_Ptr   cc_last_abbrev_endptr;
     Dwarf_Hash_Table cc_abbrev_hash_table;
+    Dwarf_Unsigned   cc_highest_known_code;
     Dwarf_CU_Context cc_next;
 
-    /*unsigned char cc_offset_length; */
-    Dwarf_Bool cc_is_info; /* TRUE means context is
+    Dwarf_Bool cc_is_info;    /* TRUE means context is
         in debug_info, FALSE means is in debug_types.
         FALSE only possible for DWARF4 .debug_types
         section CUs.
         For DWARF5 all DIEs are in .debug_info[.dwo] */
 
-    Dwarf_Half cc_unit_type;
-        /*  DWARF5: set from header
-            For DWARF 2,3,4 this is filled in initially
+    Dwarf_Half cc_unit_type; /* DWARF5
+        Set from header as a DW_UT_ value.
+        For DWARF 2,3,4 this is filled in initially
         from the CU header and refined by inspecting
         the CU DIE to detect the correct setting. */
 
@@ -318,7 +375,9 @@ struct Dwarf_Section_s {
     Dwarf_Small    dss_zdebug_requires_decompress;
     Dwarf_Small    dss_did_decompress;
     Dwarf_Small dss_shf_compressed;  /* section flag SHF_COMPRESS */
-    Dwarf_Small dss_ZLIB_compressed; /* Section compression starts with ZLIB chars*/
+
+    /* Section compression starts with ZLIB chars*/
+    Dwarf_Small dss_ZLIB_compressed;
 
     /*  For non-elf, leaving the following fields zero
         will mean they are ignored. */
@@ -328,17 +387,19 @@ struct Dwarf_Section_s {
     Dwarf_Unsigned     dss_link;
     /*  The following is used when reading .rela sections
         (such sections appear in some .o files). */
-    Dwarf_Half     dss_reloc_index; /* Zero means ignore the reloc fields. */
+    Dwarf_Half     dss_reloc_index; /* Zero means ignore
+        the reloc fields. */
     Dwarf_Small *  dss_reloc_data;
     Dwarf_Unsigned dss_reloc_size;
     Dwarf_Unsigned dss_reloc_entrysize;
     Dwarf_Addr     dss_reloc_addr;
-    /*  dss_reloc_symtab is the sh_link of a .rela to its .symtab, leave
+    /*  dss_reloc_symtab is the sh_link of a .rela
+        to its .symtab, leave
         it 0 if non-meaningful. */
     Dwarf_Addr     dss_reloc_symtab;
-    /*  dss_reloc_link should be zero unless a reloc section has a link
-        to another (sh_link).  Used to access the symtab for relocations
-        a section. */
+    /*  dss_reloc_link should be zero unless a reloc section
+        has a link to another (sh_link).
+        Used to access the symtab for relocating a section. */
     Dwarf_Unsigned     dss_reloc_link;
     /*  Pointer to the elf symtab, used for elf .rela. Leave it 0
         if not relevant. */
@@ -362,6 +423,7 @@ struct Dwarf_Section_s {
         neither compressed nor have relocations so never malloc
         space for libdwarf.  */
     Dwarf_Small     dss_ignore_reloc_group_sec;
+    char dss_is_rela;
 };
 
 /*  Overview: if next_to_use== first, no error slots are used.
@@ -383,16 +445,13 @@ struct Dwarf_Debug_InfoTypes_s {
         dwarf_next_cu_header. **Updated by dwarf_next_cu_header in
         dwarf_die_deliv.c */
     Dwarf_CU_Context de_cu_context;
-    /*  Points to linked list of CU Contexts for the CU's already read.
-        These are only CU's read by dwarf_next_cu_header(). */
+    /*  Points to linked list of CU Contexts for the
+        CU's already read.  These are only CU's read
+        by dwarf_next_cu_header(). */
     Dwarf_CU_Context de_cu_context_list;
     /*  Points to the last CU Context added to the list by
         dwarf_next_cu_header(). */
     Dwarf_CU_Context de_cu_context_list_end;
-    /*  This is the list of CU contexts read for dwarf_offdie().  These
-        may read ahead of dwarf_next_cu_header(). */
-    Dwarf_CU_Context de_offdie_cu_context;
-    Dwarf_CU_Context de_offdie_cu_context_end;
 
     /*  Offset of last byte of last CU read.
         Actually one-past that last byte.  So
@@ -409,8 +468,9 @@ struct Dwarf_Debug_InfoTypes_s {
 typedef struct Dwarf_Debug_InfoTypes_s *Dwarf_Debug_InfoTypes;
 
 /*  As the tasks performed on a debug related section is the same,
-    in order to make the process of adding a new section (very unlikely) a
-    little bit easy and to reduce the possibility of errors, a simple table
+    in order to make the process of adding a new section
+    (very unlikely) a little bit easy and to reduce the
+    possibility of errors, a simple table
     build dynamically, will contain the relevant information.
 */
 
@@ -425,13 +485,14 @@ struct Dwarf_dbg_sect_s {
     struct Dwarf_Section_s *ds_secdata;
 
     unsigned ds_groupnumber;
-    int ds_duperr;                     /* Error code for duplicated section */
-    int ds_emptyerr;                   /* Error code for empty section */
-    int ds_have_dwarf;                 /* Section contains DWARF */
-    int ds_have_zdebug;                /* Section compressed: .zdebug name */
+    int ds_duperr;            /* Error code for duplicated section */
+    int ds_emptyerr;          /* Error code for empty section */
+    int ds_have_dwarf;        /* Section contains DWARF */
+    int ds_have_zdebug;       /* Section compressed: .zdebug name */
 };
 
-/*  As the number of debug sections does not change very often, in the case a
+/*  As the number of debug sections does not change very often,
+    in the case a
     new section is added in 'enter_section_in_array()'
     the 'MAX_DEBUG_SECTIONS' must
     be updated accordingly.
@@ -439,6 +500,7 @@ struct Dwarf_dbg_sect_s {
     for which many .debug_info (and other) sections may exist.
 */
 #define DWARF_MAX_DEBUG_SECTIONS 50
+#define DWARFSTRING_ALLOC_SIZE   200
 
 
 /*  All the Dwarf_Debug tied-file info in one place.  */
@@ -511,6 +573,9 @@ struct Dwarf_Debug_s {
         under de_obj_file. */
     int  de_fd;
     char de_owns_fd;
+    /* DW_PATHSOURCE_BASIC or MACOS or DEBUGLINK */
+    unsigned char de_path_source;
+    unsigned char de_using_libelf;
     /*  de_path is only set automatically if dwarf_init_path()
         was used to initialize things.
         Used with the .gnu_debuglink section. */
@@ -545,8 +610,9 @@ struct Dwarf_Debug_s {
         sections. 4 in 32bit, 8 in MIPS 64, ia64. */
     Dwarf_Small de_pointer_size;
 
-    /*  set at creation of a Dwarf_Debug to say if form_string should be
-        checked for valid length at every call. 0 means do the check.
+    /*  set at creation of a Dwarf_Debug to say if form_string
+        should be checked for valid length at every call.
+        0 means do the check.
         non-zero means do not do the check. */
     Dwarf_Small de_assume_string_in_bounds;
 
@@ -554,16 +620,19 @@ struct Dwarf_Debug_s {
         Null till a tree is created */
     void * de_alloc_tree;
 
-    /*  These fields are used to process debug_frame section.  **Updated
+    /*  These fields are used to process debug_frame section.
+        Updated
         by dwarf_get_fde_list in dwarf_frame.h */
-    /*  Points to contiguous block of pointers to Dwarf_Cie_s structs. */
+    /*  Points to contiguous block of pointers to
+        Dwarf_Cie_s structs. */
     Dwarf_Cie *de_cie_data;
     /*  Count of number of Dwarf_Cie_s structs. */
     Dwarf_Signed de_cie_count;
     /*  Keep eh (GNU) separate!. */
     Dwarf_Cie *de_cie_data_eh;
     Dwarf_Signed de_cie_count_eh;
-    /*  Points to contiguous block of pointers to Dwarf_Fde_s structs. */
+    /*  Points to contiguous block of pointers to
+        Dwarf_Fde_s structs. */
     Dwarf_Fde *de_fde_data;
     /*  Count of number of Dwarf_Fde_s structs. */
     Dwarf_Unsigned de_fde_count;
@@ -587,15 +656,14 @@ struct Dwarf_Debug_s {
     struct Dwarf_Section_s de_debug_loclists; /* New in DWARF5 */
     struct Dwarf_Section_s de_debug_rnglists; /* New in DWARF5 */
     struct Dwarf_Section_s de_debug_frame;
-    struct Dwarf_Section_s de_gnu_debuglink;  /* New September 2019 */
-    struct Dwarf_Section_s de_note_gnu_buildid; /* New September 2019 */
-
+    struct Dwarf_Section_s de_gnu_debuglink;  /* New Sept. 2019 */
+    struct Dwarf_Section_s de_note_gnu_buildid; /* New Sept. 2019 */
 
     /* gnu: the g++ eh_frame section */
     struct Dwarf_Section_s de_debug_frame_eh_gnu;
 
-    struct Dwarf_Section_s de_debug_pubtypes; /* DWARF3 .debug_pubtypes */
-
+    /* DWARF3 .debug_pubtypes */
+    struct Dwarf_Section_s de_debug_pubtypes;
 
     /*  Four SGI IRIX extensions essentially
         identical to DWARF3 .debug_pubtypes.
@@ -610,6 +678,17 @@ struct Dwarf_Debug_s {
     struct Dwarf_Section_s de_debug_str_offsets;
     struct Dwarf_Section_s de_debug_addr;
 
+    /*  For the .debug_rnglists[.dwo] section */
+    Dwarf_Unsigned de_rnglists_context_count;
+    /*  pointer to array of pointers to
+        rnglists context instances */
+    Dwarf_Rnglists_Context *  de_rnglists_context;
+
+    /*  For the .debug_loclists[.dwo] section */
+    Dwarf_Unsigned de_loclists_context_count;
+    /*  pointer to array of pointers to
+        loclists context instances */
+    Dwarf_Loclists_Context *  de_loclists_context;
 
     /* Following for the .gdb_index section.  */
     struct Dwarf_Section_s de_debug_gdbindex;
@@ -622,9 +701,11 @@ struct Dwarf_Debug_s {
         tu and cu indexes sections are distinct in DWARF4 & 5. */
     struct Dwarf_Section_s de_debug_cu_index;
     struct Dwarf_Section_s de_debug_tu_index;
+    struct Dwarf_Section_s de_debug_gnu_pubnames;
+    struct Dwarf_Section_s de_debug_gnu_pubtypes;
 
-    /*  For non-elf, simply leave the following two structs zeroed and
-        they will be ignored. */
+    /*  For non-elf, simply leave the following two structs
+        zeroed and they will be ignored. */
     struct Dwarf_Section_s de_elf_symtab;
     struct Dwarf_Section_s de_elf_strtab;
 
@@ -667,8 +748,11 @@ struct Dwarf_Debug_s {
         See dwarf_return_empty_pubnames() */
     unsigned char de_return_empty_pubnames;
 
-    struct Dwarf_dbg_sect_s de_debug_sections[DWARF_MAX_DEBUG_SECTIONS];
-    unsigned de_debug_sections_total_entries; /* Number actually used. */
+    struct Dwarf_dbg_sect_s de_debug_sections[
+        DWARF_MAX_DEBUG_SECTIONS];
+
+    /* Number actually used. */
+    unsigned de_debug_sections_total_entries;
 
     struct Dwarf_Harmless_s de_harmless_errors;
 
@@ -678,20 +762,24 @@ struct Dwarf_Debug_s {
     /*  Used in a tied dbg  to hold global info
         on the tied object (DW_AT_dwo_id).
         And for Type Unit signatures whether tied
-        or not. */
+        or not. It is not defined whether
+        the main object is executable and
+        the tied file is a dwo/dwp or the
+        reverse. The focus of reporting
+        is on the main file, but the tied
+        file is sometimes needed
+        and referenced.*/
     struct Dwarf_Tied_Data_s de_tied_data;
-
 };
 
-int dwarf_printf(Dwarf_Debug dbg, const char * format, ...)
-#ifdef __GNUC__ /* The following gets us printf-like arg checking. */
-    __attribute__ ((format (__printf__, 2, 3)))
-#endif
-;
+/* New style. takes advantage of dwarfstrings capability.
+    This not a public function. */
+int _dwarf_printf(Dwarf_Debug dbg, const char * data);
 
 typedef struct Dwarf_Chain_s *Dwarf_Chain;
 struct Dwarf_Chain_s {
     void *ch_item;
+    int  ch_itemtype; /* Needed to dealloc chain contents */
     Dwarf_Chain ch_next;
 };
 
@@ -712,20 +800,41 @@ struct Dwarf_Chain_o {
     since DWARF3. */
 #define DISTINGUISHED_VALUE  0xffffffff
 #define DISTINGUISHED_VALUE_OFFSET_SIZE 8
-#define DISTINGUISHED_VALUE_ARRAY(x)  char x[4] = { 0xff,0xff,0xff,0xff }
+#define DISTINGUISHED_VALUE_ARRAY(x)  char x[4] = \
+    { 0xff,0xff,0xff,0xff }
 
-/*  We don't load the sections until they are needed. This function is
-    used to load the section.  */
+
+int _dwarf_ignorethissection(const char *scn_name);
+
+/*  We don't load the sections until they are needed.
+    This function is used to load the section.  */
 int _dwarf_load_section(Dwarf_Debug,
     struct Dwarf_Section_s *,
     Dwarf_Error *);
 
+void _dwarf_dealloc_rnglists_context(Dwarf_Debug dbg);
+void _dwarf_dealloc_loclists_context(Dwarf_Debug dbg);
 
 int _dwarf_get_string_base_attr_value(Dwarf_Debug dbg,
     Dwarf_CU_Context context,
     Dwarf_Unsigned *sbase_out,
     Dwarf_Error *error);
 
+int
+_dwarf_read_str_offsets_header(Dwarf_Debug dbg,
+    Dwarf_Small*     table_start_ptr,
+    Dwarf_Unsigned   secsize,
+    Dwarf_Small*     secendptr,
+    Dwarf_CU_Context cucontext,
+    /* Followed by return values/error */
+    Dwarf_Unsigned *length,
+    Dwarf_Half    *offset_size_out,
+    Dwarf_Half    *extension_size_out,
+    Dwarf_Half    *version_out,
+    Dwarf_Half    *padding_out,
+    Dwarf_Unsigned * header_length_out,
+    Dwarf_Error *error);
+
 int _dwarf_extract_string_offset_via_str_offsets(Dwarf_Debug dbg,
     Dwarf_Small *data_ptr,
     Dwarf_Small *end_data_ptr,
@@ -736,26 +845,24 @@ int _dwarf_extract_string_offset_via_str_offsets(Dwarf_Debug dbg,
     Dwarf_Error *error);
 
 
-int _dwarf_extract_address_from_debug_addr(Dwarf_Debug dbg,
+int _dwarf_look_in_local_and_tied_by_index(
+    Dwarf_Debug dbg,
     Dwarf_CU_Context context,
     Dwarf_Unsigned index,
-    Dwarf_Addr *addr_out,
+    Dwarf_Addr *return_addr,
     Dwarf_Error *error);
 
-int _dwarf_siblingof_internal(Dwarf_Debug dbg,
-    Dwarf_Die die,
-    Dwarf_CU_Context context,
-    Dwarf_Bool is_info,
-    Dwarf_Die * caller_ret_die, Dwarf_Error * error);
-
-
-int _dwarf_get_base_and_size_given_signature(Dwarf_CU_Context *context,
+#if 0
+/*  Never implemented. */
+int _dwarf_get_base_and_size_given_signature(
+    Dwarf_CU_Context *context,
     Dwarf_Sig8 *signature_in,
     /* xu_sect_index means DW_SECT_info etc. */
     Dwarf_Unsigned xu_sect_index,
     Dwarf_Unsigned *base_out,
     Dwarf_Unsigned *size_out,
     Dwarf_Error *err);
+#endif
 
 Dwarf_Bool _dwarf_file_has_debug_fission_cu_index(Dwarf_Debug dbg);
 Dwarf_Bool _dwarf_file_has_debug_fission_tu_index(Dwarf_Debug dbg);
@@ -786,15 +893,15 @@ int _dwarf_get_addr_from_tied(Dwarf_Debug dbg,
 
 
 int _dwarf_get_fission_addition_die(Dwarf_Die die, int dw_sect_index,
-   Dwarf_Unsigned* offset, Dwarf_Unsigned*size,
-   Dwarf_Error *error);
-
+    Dwarf_Unsigned* offset, Dwarf_Unsigned*size,
+    Dwarf_Error *error);
 int _dwarf_get_addr_index_itself(int theform,
     Dwarf_Small *info_ptr,
     Dwarf_Debug dbg,
     Dwarf_CU_Context cu_context,
     Dwarf_Unsigned *val_out,
     Dwarf_Error * error);
+Dwarf_Bool _dwarf_addr_form_is_indexed(int form);
 
 int
 _dwarf_search_for_signature(Dwarf_Debug dbg,
@@ -802,6 +909,12 @@ _dwarf_search_for_signature(Dwarf_Debug dbg,
    Dwarf_CU_Context *context_out,
    Dwarf_Error *error);
 
+int
+_dwarf_merge_all_base_attrs_of_cu_die(Dwarf_Debug dbg,
+    Dwarf_CU_Context context,
+    Dwarf_Debug tieddbg,
+    Dwarf_CU_Context *tiedcontext_out,
+    Dwarf_Error *error);
 
 void _dwarf_tied_destroy_free_node(void *node);
 void _dwarf_destroy_group_map(Dwarf_Debug dbg);
@@ -861,12 +974,14 @@ int _dwarf_get_ranges_base_attr_from_tied(Dwarf_Debug dbg,
     Dwarf_Unsigned * addr_base_out,
     Dwarf_Error    * error);
 
-int _dwarf_get_string_from_tied(Dwarf_Debug dbg, Dwarf_Unsigned offset,
+int _dwarf_get_string_from_tied(Dwarf_Debug dbg,
+    Dwarf_Unsigned offset,
     char **return_str, Dwarf_Error*error);
 
 int _dwarf_valid_form_we_know(Dwarf_Unsigned at_form,
     Dwarf_Unsigned at_name);
-int _dwarf_extract_local_debug_str_string_given_offset(Dwarf_Debug dbg,
+int _dwarf_extract_local_debug_str_string_given_offset(
+    Dwarf_Debug dbg,
     unsigned attrform,
     Dwarf_Unsigned offset,
     char ** return_str,
@@ -884,7 +999,8 @@ typedef int (*_dwarf_get_elf_flags_func_ptr_type)(
     Dwarf_Unsigned *flags_out,
     Dwarf_Unsigned *addralign_out,
     int *error);
-extern _dwarf_get_elf_flags_func_ptr_type _dwarf_get_elf_flags_func_ptr;
+extern _dwarf_get_elf_flags_func_ptr_type
+    _dwarf_get_elf_flags_func_ptr;
 
 /* This is libelf access to Elf object. */
 extern int _dwarf_elf_setup(int fd,
@@ -912,7 +1028,8 @@ _dwarf_elf_nlsetup(int fd,
     Dwarf_Handler errhand,
     Dwarf_Ptr errarg,
     Dwarf_Debug *dbg,Dwarf_Error *error);
-void _dwarf_destruct_elf_nlaccess(struct Dwarf_Obj_Access_Interface_s *aip);
+void _dwarf_destruct_elf_nlaccess(
+    struct Dwarf_Obj_Access_Interface_s *aip);
 
 extern int _dwarf_macho_setup(int fd,
     char *true_path,
@@ -925,7 +1042,8 @@ extern int _dwarf_macho_setup(int fd,
     Dwarf_Handler errhand,
     Dwarf_Ptr errarg,
     Dwarf_Debug *dbg,Dwarf_Error *error);
-void _dwarf_destruct_macho_access(struct Dwarf_Obj_Access_Interface_s *aip);
+void _dwarf_destruct_macho_access(
+    struct Dwarf_Obj_Access_Interface_s *aip);
 
 extern int _dwarf_pe_setup(int fd,
     char *path,
@@ -938,7 +1056,14 @@ extern int _dwarf_pe_setup(int fd,
     Dwarf_Handler errhand,
     Dwarf_Ptr errarg,
     Dwarf_Debug *dbg,Dwarf_Error *error);
-void _dwarf_destruct_pe_access(struct Dwarf_Obj_Access_Interface_s *aip);
+void _dwarf_destruct_pe_access(
+    struct Dwarf_Obj_Access_Interface_s *aip);
+
+void _dwarf_create_address_size_dwarf_error(Dwarf_Debug dbg,
+    Dwarf_Error *error,
+    Dwarf_Unsigned addrsize,
+    int errcode,const char *errname);
+
 
 extern Dwarf_Bool _dwarf_allow_formudata(unsigned form);
 extern int _dwarf_formudata_internal(Dwarf_Debug dbg,
@@ -949,10 +1074,20 @@ extern int _dwarf_formudata_internal(Dwarf_Debug dbg,
     Dwarf_Unsigned *bytes_read,
     Dwarf_Error *error);
 
-Dwarf_Byte_Ptr _dwarf_calculate_info_section_start_ptr(Dwarf_CU_Context context, Dwarf_Unsigned *section_len_out);
+Dwarf_Byte_Ptr _dwarf_calculate_info_section_start_ptr(
+    Dwarf_CU_Context context,
+    Dwarf_Unsigned *section_len_out);
 
-Dwarf_Byte_Ptr _dwarf_calculate_info_section_end_ptr(Dwarf_CU_Context context);
-Dwarf_Byte_Ptr _dwarf_calculate_abbrev_section_end_ptr(Dwarf_CU_Context context);
+Dwarf_Byte_Ptr _dwarf_calculate_info_section_end_ptr(
+    Dwarf_CU_Context context);
+Dwarf_Byte_Ptr _dwarf_calculate_abbrev_section_end_ptr(
+    Dwarf_CU_Context context);
+
+int _dwarf_formblock_internal(Dwarf_Debug dbg,
+    Dwarf_Attribute attr,
+    Dwarf_CU_Context cu_context,
+    Dwarf_Block * return_block,
+    Dwarf_Error * error);
 
 int _dwarf_extract_data16(Dwarf_Debug dbg,
     Dwarf_Small *data,
@@ -961,5 +1096,9 @@ int _dwarf_extract_data16(Dwarf_Debug dbg,
     Dwarf_Form_Data16  * returned_val,
     Dwarf_Error *error);
 
+unsigned int  _dwarf_crc32(unsigned int init,
+    const unsigned char * buf,
+    size_t len);
 
-void _dwarf_dumpsig(const char *msg, Dwarf_Sig8 *sig,int lineno);
+
+void _dwarf_dumpsig(const char *msg, Dwarf_Sig8 *sig, int lineno);
