@@ -1,27 +1,31 @@
 /*
   Copyright (C) 2000,2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
-  Portions Copyright (C) 2007-2018 David Anderson. All Rights Reserved.
+  Portions Copyright (C) 2007-2020 David Anderson. All Rights Reserved.
   Portions Copyright 2012 SN Systems Ltd. All rights reserved.
+  Portions Copyright 2020 Google All rights reserved.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of version 2.1 of the GNU Lesser General Public License
-  as published by the Free Software Foundation.
+  This program is free software; you can redistribute it
+  and/or modify it under the terms of version 2.1 of the
+  GNU Lesser General Public License as published by the Free
+  Software Foundation.
 
-  This program is distributed in the hope that it would be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  This program is distributed in the hope that it would be
+  useful, but WITHOUT ANY WARRANTY; without even the implied
+  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+  PURPOSE.
 
-  Further, this software is distributed without any warranty that it is
-  free of the rightful claim of any third person regarding infringement
-  or the like.  Any license provided herein, whether implied or
-  otherwise, applies only to this software file.  Patent licenses, if
-  any, provided herein do not apply to combinations of this program with
-  other software, or any other product whatsoever.
+  Further, this software is distributed without any warranty
+  that it is free of the rightful claim of any third person
+  regarding infringement or the like.  Any license provided
+  herein, whether implied or otherwise, applies only to this
+  software file.  Patent licenses, if any, provided herein
+  do not apply to combinations of this program with other
+  software, or any other product whatsoever.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this program; if not, write the Free Software
-  Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston MA 02110-1301,
-  USA.
+  You should have received a copy of the GNU Lesser General
+  Public License along with this program; if not, write the
+  Free Software Foundation, Inc., 51 Franklin Street - Fifth
+  Floor, Boston MA 02110-1301, USA.
 
 */
 
@@ -32,6 +36,7 @@
 #include "dwarf_error.h"
 #include "dwarf_util.h"
 #include "dwarf_die_deliv.h"
+#include "dwarfstring.h"
 
 #define TRUE 1
 static int _dwarf_die_attr_unsigned_constant(Dwarf_Die die,
@@ -39,16 +44,6 @@ static int _dwarf_die_attr_unsigned_constant(Dwarf_Die die,
     Dwarf_Unsigned * return_val,
     Dwarf_Error * error);
 
-static int _dwarf_get_ranges_base_attr_value(Dwarf_Debug dbg,
-    Dwarf_CU_Context context,
-    Dwarf_Unsigned * rabase_out,
-    Dwarf_Error    * error);
-
-static int _dwarf_get_address_base_attr_value(Dwarf_Debug dbg,
-    Dwarf_CU_Context context,
-    Dwarf_Unsigned *abase_out,
-    Dwarf_Error *error);
-
 int dwarf_get_offset_size(Dwarf_Debug dbg,
     Dwarf_Half  *    offset_size,
     Dwarf_Error *    error)
@@ -76,8 +71,6 @@ dump_bytes(char * msg,Dwarf_Small * start, long len)
 }
 #endif
 
-
-
 /* This is normally reliable.
 But not always.
 If different compilation
@@ -167,12 +160,22 @@ dwarf_die_offsets(Dwarf_Die die,
     Dwarf_Error *error)
 {
     int res = 0;
+    Dwarf_Off lcuoff = 0;
+    Dwarf_Off loff = 0;
 
-    *off = 0;
-    *cu_off = 0;
-    res = dwarf_dieoffset(die,off,error);
+    res = dwarf_dieoffset(die,&loff,error);
     if (res == DW_DLV_OK) {
-        res = dwarf_die_CU_offset(die,cu_off,error);
+        res = dwarf_die_CU_offset(die,&lcuoff,error);
+    }
+    if (res == DW_DLV_OK) {
+        /*  Waiting till both succeed before
+            returning any value at all to retain
+            normal libdwarf call semantics. */
+        *off = loff;
+        *cu_off = lcuoff;
+    } else {
+        *off = 0;
+        *cu_off = 0;
     }
     return res;
 }
@@ -306,14 +309,14 @@ dwarf_offset_list(Dwarf_Debug dbg,
     }
 
     /* Points to contiguous block of Dwarf_Off's. */
-    ret_offsets = (Dwarf_Off *) _dwarf_get_alloc(dbg, 
+    ret_offsets = (Dwarf_Off *) _dwarf_get_alloc(dbg,
         DW_DLA_ADDR, off_count);
     if (ret_offsets == NULL) {
         _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
         return (DW_DLV_ERROR);
     }
 
-    /*  Store offsets in contiguous block, 
+    /*  Store offsets in contiguous block,
         and deallocate the chain. */
     curr_chain = head_chain;
     for (i = 0; i < off_count; i++) {
@@ -329,9 +332,21 @@ dwarf_offset_list(Dwarf_Debug dbg,
     return DW_DLV_OK;
 }
 
-/*  If the input is improper (see DW_DLV_ERROR)
-    this may leak memory. Such badly formed input
-    should be very very rare.
+static void
+empty_local_attrlist(Dwarf_Debug dbg,
+    Dwarf_Attribute attr)
+{
+    Dwarf_Attribute cur = 0;
+    Dwarf_Attribute next = 0;
+
+    for (cur = attr; cur ; cur = next) {
+        next = cur->ar_next;
+        dwarf_dealloc(dbg,cur,DW_DLA_ATTR);
+    }
+}
+
+/*  Now we use *_wrapper here,
+    We cannot leak memory.
 */
 int
 dwarf_attrlist(Dwarf_Die die,
@@ -339,13 +354,12 @@ dwarf_attrlist(Dwarf_Die die,
     Dwarf_Signed * attrcnt, Dwarf_Error * error)
 {
     Dwarf_Unsigned attr_count = 0;
+    Dwarf_Unsigned attr = 0;
+    Dwarf_Unsigned attr_form = 0;
     Dwarf_Unsigned i = 0;
-    Dwarf_Half attr = 0;
-    Dwarf_Half attr_form = 0;
     Dwarf_Byte_Ptr abbrev_ptr = 0;
     Dwarf_Byte_Ptr abbrev_end = 0;
     Dwarf_Abbrev_List abbrev_list = 0;
-    Dwarf_Attribute new_attr = 0;
     Dwarf_Attribute head_attr = NULL;
     Dwarf_Attribute curr_attr = NULL;
     Dwarf_Attribute *attr_ptr = 0;
@@ -354,6 +368,7 @@ dwarf_attrlist(Dwarf_Die die,
     Dwarf_Byte_Ptr die_info_end = 0;
     int lres = 0;
     Dwarf_CU_Context context = 0;
+    Dwarf_Unsigned highest_code = 0;
 
     CHECK_DIE(die, DW_DLV_ERROR);
     context = die->di_cu_context;
@@ -363,12 +378,28 @@ dwarf_attrlist(Dwarf_Die die,
 
     lres = _dwarf_get_abbrev_for_code(context,
         die->di_abbrev_list->abl_code,
-        &abbrev_list,error);
+        &abbrev_list,
+        &highest_code,error);
     if (lres == DW_DLV_ERROR) {
         return lres;
     }
     if (lres == DW_DLV_NO_ENTRY) {
-        _dwarf_error(dbg, error, DW_DLE_ABBREV_MISSING);
+        dwarfstring m;
+
+        dwarfstring_constructor(&m);
+        dwarfstring_append_printf_u(&m,
+            "DW_DLE_ABBREV_MISSING "
+            "There is no abbrev present for code %u "
+            "in this compilation unit. ",
+            die->di_abbrev_list->abl_code);
+        dwarfstring_append_printf_u(&m,
+            "The highest known code "
+            "in any compilation unit is %u .",
+            highest_code);
+        _dwarf_error_string(dbg, error,
+            DW_DLE_ABBREV_MISSING,
+            dwarfstring_string(&m));
+        dwarfstring_destructor(&m);
         return DW_DLV_ERROR;
     }
 
@@ -377,62 +408,115 @@ dwarf_attrlist(Dwarf_Die die,
 
 
     info_ptr = die->di_debug_ptr;
-    SKIP_LEB128_WORD_CK(info_ptr,dbg,error,die_info_end);
-    if (info_ptr >= die_info_end) {
-        /* Stepped off the end SKIPping the leb  */
-        _dwarf_error(dbg, error, DW_DLE_DIE_BAD);
-        return DW_DLV_ERROR;
+    {
+        /* SKIP_LEB128 */
+        Dwarf_Unsigned ignore_this = 0;
+        Dwarf_Unsigned len = 0;
+
+        lres = _dwarf_decode_u_leb128_chk(info_ptr,
+            &len,&ignore_this,die_info_end);
+        if (lres == DW_DLV_ERROR) {
+            /* Stepped off the end SKIPping the leb  */
+            dwarfstring m;
+
+            dwarfstring_constructor(&m);
+            dwarfstring_append_printf_u(&m,
+                "DW_DLE_DIE_BAD: In building an attrlist "
+                "we run off the end of the DIE while skipping "
+                " the DIE tag, seeing the leb length as 0x%u ",
+                len);
+            _dwarf_error_string(dbg, error, DW_DLE_DIE_BAD,
+                dwarfstring_string(&m));
+            dwarfstring_destructor(&m);
+            return DW_DLV_ERROR;
+        }
+        info_ptr += len;
     }
 
     do {
-        Dwarf_Unsigned utmp2;
         Dwarf_Signed implicit_const = 0;
+        Dwarf_Attribute new_attr = 0;
+        int res = 0;
 
-        DECODE_LEB128_UWORD_CK(abbrev_ptr, utmp2,dbg,error,abbrev_end);
-        if (utmp2 > DW_AT_hi_user) {
+        /*  The DECODE have to be wrapped in functions to
+            catch errors before return. */
+        /*DECODE_LEB128_UWORD_CK(abbrev_ptr, utmp2,
+            dbg,error,abbrev_end); */
+        res = _dwarf_leb128_uword_wrapper(dbg,
+            &abbrev_ptr,abbrev_end,&attr,error);
+        if (res == DW_DLV_ERROR) {
+            empty_local_attrlist(dbg,head_attr);
+            return res;
+        }
+        if (attr > DW_AT_hi_user) {
+            empty_local_attrlist(dbg,head_attr);
             _dwarf_error(dbg, error,DW_DLE_ATTR_CORRUPT);
             return DW_DLV_ERROR;
         }
-        attr = (Dwarf_Half) utmp2;
-        DECODE_LEB128_UWORD_CK(abbrev_ptr, utmp2,dbg,error,abbrev_end);
-        if (!_dwarf_valid_form_we_know(utmp2,attr)) {
+        /*DECODE_LEB128_UWORD_CK(abbrev_ptr, utmp2,
+            dbg,error,abbrev_end); */
+        res = _dwarf_leb128_uword_wrapper(dbg,
+            &abbrev_ptr,abbrev_end,&attr_form,error);
+        if (res == DW_DLV_ERROR) {
+            empty_local_attrlist(dbg,head_attr);
+            return res;
+        }
+        if (!_dwarf_valid_form_we_know(attr_form,attr)) {
+            empty_local_attrlist(dbg,head_attr);
             _dwarf_error(dbg, error, DW_DLE_UNKNOWN_FORM);
             return (DW_DLV_ERROR);
         }
-        attr_form = (Dwarf_Half) utmp2;
         if (attr_form == DW_FORM_implicit_const) {
             /* The value is here, not in a DIE. */
-            DECODE_LEB128_SWORD_CK(abbrev_ptr, implicit_const,
-                dbg,error,abbrev_end);
+            res = _dwarf_leb128_sword_wrapper(dbg,&abbrev_ptr,
+                abbrev_end, &implicit_const, error);
+            if (res == DW_DLV_ERROR) {
+                empty_local_attrlist(dbg,head_attr);
+                return res;
+            }
+            /*DECODE_LEB128_SWORD_CK(abbrev_ptr, implicit_const,
+                dbg,error,abbrev_end); */
         }
 
         if (!_dwarf_valid_form_we_know(attr_form,attr)) {
+            empty_local_attrlist(dbg,head_attr);
             _dwarf_error(dbg, error, DW_DLE_UNKNOWN_FORM);
             return DW_DLV_ERROR;
         }
-
         if (attr != 0) {
-            new_attr =
-                (Dwarf_Attribute) _dwarf_get_alloc(dbg, DW_DLA_ATTR, 1);
+            new_attr = (Dwarf_Attribute)
+                _dwarf_get_alloc(dbg, DW_DLA_ATTR, 1);
             if (new_attr == NULL) {
+                empty_local_attrlist(dbg,head_attr);
                 _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
                 return DW_DLV_ERROR;
             }
-
             new_attr->ar_attribute = attr;
             new_attr->ar_attribute_form_direct = attr_form;
             new_attr->ar_attribute_form = attr_form;
             if (attr_form == DW_FORM_indirect) {
-                Dwarf_Unsigned utmp6;
+                Dwarf_Unsigned utmp6 = 0;
+
                 if (_dwarf_reference_outside_section(die,
                     (Dwarf_Small*) info_ptr,
                     ((Dwarf_Small*) info_ptr )+1)) {
-                    _dwarf_error(dbg, error,DW_DLE_ATTR_OUTSIDE_SECTION);
+                    dwarf_dealloc(dbg,new_attr,DW_DLA_ATTR);
+                    empty_local_attrlist(dbg,head_attr);
+                    _dwarf_error_string(dbg, error,
+                        DW_DLE_ATTR_OUTSIDE_SECTION,
+                        "DW_DLE_ATTR_OUTSIDE_SECTION: "
+                        " Reading Attriutes: "
+                        "For DW_FORM_indirect there is"
+                        " no room for the form. Corrupt Dwarf");
                     return DW_DLV_ERROR;
                 }
 
-                /* DECODE_LEB128_UWORD does info_ptr update */
-                DECODE_LEB128_UWORD_CK(info_ptr, utmp6,dbg,error,die_info_end);
+                /*  DECODE_LEB128_UWORD does info_ptr update
+                    DECODE_LEB128_UWORD_CK(info_ptr, utmp6,
+                        dbg,error,die_info_end);
+                */
+                res = _dwarf_leb128_uword_wrapper(dbg,
+                    &info_ptr,die_info_end,&utmp6,error);
                 attr_form = (Dwarf_Half) utmp6;
                 new_attr->ar_attribute_form = attr_form;
             }
@@ -445,21 +529,29 @@ dwarf_attrlist(Dwarf_Die die,
                 _dwarf_reference_outside_section(die,
                 (Dwarf_Small*) info_ptr,
                 ((Dwarf_Small*) info_ptr )+1)) {
-                _dwarf_error(dbg, error,DW_DLE_ATTR_OUTSIDE_SECTION);
+                dwarf_dealloc(dbg,new_attr,DW_DLA_ATTR);
+                empty_local_attrlist(dbg,head_attr);
+                _dwarf_error_string(dbg, error,
+                    DW_DLE_ATTR_OUTSIDE_SECTION,
+                    "DW_DLE_ATTR_OUTSIDE_SECTION: "
+                    " Reading Attriutes: "
+                    "We have run off the end of the section. "
+                    "Corrupt Dwarf");
                 return DW_DLV_ERROR;
             }
             new_attr->ar_cu_context = die->di_cu_context;
             new_attr->ar_debug_ptr = info_ptr;
             new_attr->ar_die = die;
+            new_attr->ar_dbg = dbg;
             if (attr_form == DW_FORM_implicit_const) {
                 /*  The value is here, not in a DIE.
                     Do not increment info_ptr */
                 new_attr->ar_implicit_const = implicit_const;
             } else {
                 Dwarf_Unsigned sov = 0;
-                int res = 0;
+                int vres = 0;
 
-                res = _dwarf_get_size_of_val(dbg,
+                vres = _dwarf_get_size_of_val(dbg,
                     attr_form,
                     die->di_cu_context->cc_version_stamp,
                     die->di_cu_context->cc_address_size,
@@ -468,12 +560,13 @@ dwarf_attrlist(Dwarf_Die die,
                     &sov,
                     die_info_end,
                     error);
-                if(res!= DW_DLV_OK) {
-                    return res;
+                if(vres!= DW_DLV_OK) {
+                    dwarf_dealloc(dbg,new_attr,DW_DLA_ATTR);
+                    empty_local_attrlist(dbg,head_attr);
+                    return vres;
                 }
                 info_ptr += sov;
             }
-
             if (head_attr == NULL)
                 head_attr = curr_attr = new_attr;
             else {
@@ -482,27 +575,24 @@ dwarf_attrlist(Dwarf_Die die,
             }
             attr_count++;
         }
-    } while (attr != 0 || attr_form != 0);
-
-    if (attr_count == 0) {
+    } while (attr || attr_form);
+    if (!attr_count) {
         *attrbuf = NULL;
         *attrcnt = 0;
         return (DW_DLV_NO_ENTRY);
     }
-
     attr_ptr = (Dwarf_Attribute *)
         _dwarf_get_alloc(dbg, DW_DLA_LIST, attr_count);
     if (attr_ptr == NULL) {
+        empty_local_attrlist(dbg,head_attr);
         _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
         return (DW_DLV_ERROR);
     }
-
     curr_attr = head_attr;
     for (i = 0; i < attr_count; i++) {
         *(attr_ptr + i) = curr_attr;
         curr_attr = curr_attr->ar_next;
     }
-
     *attrbuf = attr_ptr;
     *attrcnt = attr_count;
     return (DW_DLV_OK);
@@ -542,6 +632,7 @@ _dwarf_get_value_ptr(Dwarf_Die die,
     Dwarf_Byte_Ptr die_info_end = 0;
     Dwarf_Debug dbg = 0;
     int lres = 0;
+    Dwarf_Unsigned highest_code = 0;
 
     if (!context) {
         _dwarf_error(NULL,error,DW_DLE_DIE_NO_CU_CONTEXT);
@@ -553,12 +644,27 @@ _dwarf_get_value_ptr(Dwarf_Die die,
 
     lres = _dwarf_get_abbrev_for_code(context,
         die->di_abbrev_list->abl_code,
-        &abbrev_list,error);
+        &abbrev_list,&highest_code,error);
     if (lres == DW_DLV_ERROR) {
         return lres;
     }
     if (lres == DW_DLV_NO_ENTRY) {
-        _dwarf_error(dbg,error,DW_DLE_CU_DIE_NO_ABBREV_LIST);
+        dwarfstring m;
+
+        dwarfstring_constructor(&m);
+        dwarfstring_append_printf_u(&m,
+            "DW_DLE_CU_DIE_NO_ABBREV_LIST "
+            "There is no abbrev present for code %u "
+            "in this compilation unit. ",
+            die->di_abbrev_list->abl_code);
+        dwarfstring_append_printf_u(&m,
+            "The highest known code "
+            "in any compilation unit is %u.",
+            highest_code);
+        _dwarf_error_string(dbg, error,
+            DW_DLE_CU_DIE_NO_ABBREV_LIST,
+            dwarfstring_string(&m));
+        dwarfstring_destructor(&m);
         return DW_DLV_ERROR;
     }
 
@@ -567,8 +673,30 @@ _dwarf_get_value_ptr(Dwarf_Die die,
 
     info_ptr = die->di_debug_ptr;
     /* This ensures and checks die_info_end >= info_ptr */
-    SKIP_LEB128_WORD_CK(info_ptr,dbg,error,die_info_end);
+    {
+        /* SKIP_LEB128 */
+        Dwarf_Unsigned ignore_this = 0;
+        Dwarf_Unsigned len = 0;
 
+        lres = _dwarf_decode_u_leb128_chk(info_ptr,
+            &len,&ignore_this,die_info_end);
+        if (lres == DW_DLV_ERROR) {
+            /* Stepped off the end SKIPping the leb  */
+            dwarfstring m;
+
+            dwarfstring_constructor(&m);
+            dwarfstring_append_printf_u(&m,
+                "DW_DLE_DIE_BAD: In building an attrlist "
+                "we run off the end of the DIE while skipping "
+                " the DIE tag, seeing the leb length as 0x%u ",
+                len);
+            _dwarf_error_string(dbg, error, DW_DLE_DIE_BAD,
+                dwarfstring_string(&m));
+            dwarfstring_destructor(&m);
+            return DW_DLV_ERROR;
+        }
+        info_ptr += len;
+    }
     do {
         Dwarf_Unsigned formtmp3 = 0;
         Dwarf_Unsigned atmp3 = 0;
@@ -624,9 +752,12 @@ _dwarf_get_value_ptr(Dwarf_Die die,
             return res;
         }
         {
-            /* ptrdiff_t is signed type, so use DW signed type */
-            Dwarf_Signed len = die_info_end - info_ptr;
-            if (len < 0 || (value_size > ((Dwarf_Unsigned)len))) {
+            Dwarf_Unsigned len  = 0;
+
+            /*  ptrdiff_t is generated but not named */
+            len = (die_info_end >= info_ptr)?
+                (die_info_end - info_ptr):0;
+            if (value_size > len) {
                 /*  Something badly wrong. We point past end
                     of debug_info or debug_types or a
                     section is unreasonably sized or we are
@@ -725,9 +856,11 @@ dwarf_attr(Dwarf_Die die,
     }
 
     attrib = (Dwarf_Attribute) _dwarf_get_alloc(dbg, DW_DLA_ATTR, 1);
-    if (attrib == NULL) {
-        _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
-        return (DW_DLV_ERROR);
+    if (!attrib) {
+        _dwarf_error_string(dbg, error, DW_DLE_ALLOC_FAIL,
+            "DW_DLE_ALLOC_FAIL allocating a single Dwarf_Attribute"
+            " in function dwarf_attr().");
+        return DW_DLV_ERROR;
     }
 
     attrib->ar_attribute = attr;
@@ -740,14 +873,20 @@ dwarf_attr(Dwarf_Die die,
     /*  Only nonnull if not DW_FORM_implicit_const */
     attrib->ar_debug_ptr = info_ptr;
     attrib->ar_die = die;
+    attrib->ar_dbg = dbg;
     *ret_attr = (attrib);
     return DW_DLV_OK;
 }
 
 /*  A DWP (.dwp) package object never contains .debug_addr,
     only a normal .o or executable object.
-    Error returned here is on dbg, not tieddbg. */
-int
+    Error returned here is on dbg, not tieddbg.
+    This looks for DW_AT_addr_base and if present
+    adds it in appropriately.
+    You should use _dwarf_look_in_local_and_tied_by_index()
+    instead of this, in general.
+    */
+static int
 _dwarf_extract_address_from_debug_addr(Dwarf_Debug dbg,
     Dwarf_CU_Context context,
     Dwarf_Unsigned index_to_addr,
@@ -763,11 +902,7 @@ _dwarf_extract_address_from_debug_addr(Dwarf_Debug dbg,
     Dwarf_Byte_Ptr  sectionend = 0;
     Dwarf_Unsigned  sectionsize  = 0;
 
-    res = _dwarf_get_address_base_attr_value(dbg,context,
-        &address_base, error);
-    if (res != DW_DLV_OK) {
-        return res;
-    }
+    address_base = context->cc_addr_base;
     res = _dwarf_load_section(dbg, &dbg->de_debug_addr,error);
     if (res != DW_DLV_OK) {
         /*  Ignore the inner error, report something meaningful */
@@ -792,7 +927,23 @@ _dwarf_extract_address_from_debug_addr(Dwarf_Debug dbg,
     sectionsize = dbg->de_debug_addr.dss_size;
     sectionend = sectionstart + sectionsize;
     if (addr_offset > (sectionsize - context->cc_address_size)) {
-        _dwarf_error(dbg, error, DW_DLE_ATTR_FORM_SIZE_BAD);
+        dwarfstring m;
+
+        dwarfstring_constructor(&m);
+        dwarfstring_append_printf_u(&m,
+            "DW_DLE_ATTR_FORM_SIZE_BAD: "
+            "Extracting an address from .debug_addr fails"
+            "as the offset is  0x%x ",
+            addr_offset);
+        dwarfstring_append_printf_u(&m,
+            "but the object section is just 0x%x "
+            "bytes long so there not enough space"
+            " for an address.",
+            sectionsize);
+        _dwarf_error_string(dbg, error,
+            DW_DLE_ATTR_FORM_SIZE_BAD,
+            dwarfstring_string(&m));
+        dwarfstring_destructor(&m);
         return (DW_DLV_ERROR);
     }
     READ_UNALIGNED_CK(dbg,ret_addr,Dwarf_Addr,
@@ -803,7 +954,7 @@ _dwarf_extract_address_from_debug_addr(Dwarf_Debug dbg,
     return DW_DLV_OK;
 }
 
-static int
+int
 _dwarf_look_in_local_and_tied_by_index(
     Dwarf_Debug dbg,
     Dwarf_CU_Context context,
@@ -818,27 +969,22 @@ _dwarf_look_in_local_and_tied_by_index(
     if (res2 != DW_DLV_OK) {
         if (res2 == DW_DLV_ERROR &&
             error &&
-            dwarf_errno(*error) == DW_DLE_MISSING_NEEDED_DEBUG_ADDR_SECTION
+            dwarf_errno(*error) ==
+            DW_DLE_MISSING_NEEDED_DEBUG_ADDR_SECTION
             && dbg->de_tied_data.td_tied_object) {
             int res3 = 0;
 
             /*  We do not want to leak error structs... */
             dwarf_dealloc(dbg,*error,DW_DLA_ERROR);
-
             *error = 0;
             /* error is returned on dbg, not tieddbg. */
             res3 = _dwarf_get_addr_from_tied(dbg,
                 context,index,return_addr,error);
-            if ( res3 == DW_DLV_ERROR) {
-                return res3;
-            } else if ( res3 == DW_DLV_NO_ENTRY) {
-                return res3;
-            }
-        } else {
-            return res2;
+            return res3;
         }
+        return res2;
     }
-    return (DW_DLV_OK);
+    return DW_DLV_OK;
 }
 
 /*  The DIE here can be any DIE in the relevant CU.
@@ -997,72 +1143,96 @@ dwarf_dietype_offset(Dwarf_Die die,
     res = dwarf_attr(die,DW_AT_type,&attr,error);
     if (res == DW_DLV_OK) {
         res = dwarf_global_formref(attr,&offset,error);
-        dwarf_dealloc(die->di_cu_context->cc_dbg,attr,DW_DLA_ATTR);
+        dwarf_dealloc(die->di_cu_context->cc_dbg,attr,
+            DW_DLA_ATTR);
     }
     *return_off = offset;
     return res;
 }
 
+int
+_dwarf_merge_all_base_attrs_of_cu_die(Dwarf_Debug dbg,
+    Dwarf_CU_Context context,
+    Dwarf_Debug tieddbg,
+    Dwarf_CU_Context *tiedcontext_out,
+    Dwarf_Error *error)
+{
+    Dwarf_CU_Context tiedcontext = 0;
+    int res = 0;
+
+    if (!tieddbg) {
+        return DW_DLV_NO_ENTRY;
+    }
+    if (!context->cc_signature_present) {
+        return DW_DLV_NO_ENTRY;
+    }
+    res = _dwarf_search_for_signature(tieddbg,
+        context->cc_signature,
+        &tiedcontext,
+        error);
+    if ( res == DW_DLV_ERROR) {
+        /* Associate the error with dbg, not tieddbg */
+        _dwarf_error_mv_s_to_t(tieddbg,error,dbg,error);
+        return res;
+    } else if ( res == DW_DLV_NO_ENTRY) {
+        return res;
+    }
+    if (!context->cc_low_pc_present) {
+        context->cc_low_pc_present =
+            tiedcontext->cc_low_pc_present;
+        context->        cc_low_pc=
+            tiedcontext->cc_low_pc;
+    }
+    if (!context->cc_addr_base_present) {
+        context->        cc_addr_base_present =
+            tiedcontext->cc_addr_base_present;
+        context->        cc_addr_base=
+            tiedcontext->cc_addr_base;
+    }
+    if (!context->cc_rnglists_base_present) {
+        context->        cc_rnglists_base_present =
+            tiedcontext->cc_rnglists_base_present;
+        context->        cc_rnglists_base=
+            tiedcontext->cc_rnglists_base;
+    }
+    if(!context->cc_loclists_base_present) {
+        context->        cc_loclists_base_present =
+            tiedcontext->cc_loclists_base_present;
+        context->        cc_loclists_base=
+            tiedcontext->cc_loclists_base;
+    }
+    if(!context->cc_str_offsets_base_present) {
+        context->        cc_str_offsets_base_present =
+            tiedcontext->cc_str_offsets_base_present;
+        context->        cc_str_offsets_base=
+            tiedcontext->cc_str_offsets_base;
+    }
+
+    /* GNU DW4 extension. */
+    if(!context->        cc_ranges_base_present) {
+        context->        cc_ranges_base_present =
+            tiedcontext->cc_ranges_base_present;
+        context->        cc_ranges_base =
+            tiedcontext->cc_ranges_base;
+    }
+
+    if(tiedcontext_out) {
+        *tiedcontext_out = tiedcontext;
+    }
+    return DW_DLV_OK;
+}
 
 
 int
-_dwarf_get_string_base_attr_value(Dwarf_Debug dbg,
+_dwarf_get_string_base_attr_value(UNUSEDARG Dwarf_Debug dbg,
     Dwarf_CU_Context context,
     Dwarf_Unsigned *sbase_out,
-    Dwarf_Error *error)
+    UNUSEDARG Dwarf_Error *error)
 {
-    int res = 0;
-    Dwarf_Die cudie = 0;
-    Dwarf_Unsigned cu_die_offset = 0;
-    Dwarf_Attribute myattr = 0;
-
     if(context->cc_str_offsets_base_present) {
         *sbase_out = context->cc_str_offsets_base;
         return DW_DLV_OK;
     }
-    cu_die_offset = context->cc_cu_die_global_sec_offset;
-    context->cc_cu_die_offset_present  = TRUE;
-    res = dwarf_offdie_b(dbg,cu_die_offset,
-        context->cc_is_info,
-        &cudie,
-        error);
-    if(res != DW_DLV_OK) {
-        return res;
-    }
-    res = dwarf_attr(cudie,DW_AT_str_offsets_base,
-        &myattr,error);
-    if(res == DW_DLV_ERROR) {
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        return res;
-    }
-    if (res == DW_DLV_OK) {
-        Dwarf_Unsigned val = 0;
-        /* Expect DW_FORM_sec_offset */
-        if (myattr->ar_attribute_form != DW_FORM_sec_offset) {
-            dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-            dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-            _dwarf_error(dbg, error,DW_DLE_STR_OFFSETS_BASE_WRONG_FORM);
-            return (DW_DLV_ERROR);
-        }
-        res = dwarf_global_formref(myattr,&val,error);
-        dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        if(res != DW_DLV_OK) {
-            return res;
-        }
-        *sbase_out  = val;
-        context->cc_str_offsets_base = val;
-        context->cc_str_offsets_base_present = TRUE;
-        return DW_DLV_OK;
-    }
-    /*  NO ENTRY, No other attr.Not even GNU, this one is standard
-        DWARF5 only.  */
-    dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-    /*  We do not need a base for a .dwo. We might for .dwp
-        and would or .o or executable.
-        FIXME: assume we do not need this.
-        Should we really return DW_DLV_NO_ENTRY?
-    */
     *sbase_out = 0;
     return DW_DLV_OK;
 }
@@ -1071,172 +1241,8 @@ _dwarf_get_string_base_attr_value(Dwarf_Debug dbg,
     and returns it through abase_out. If we cannot find the value
     it is a serious error in the DWARF.
     */
-static int
-_dwarf_get_address_base_attr_value(Dwarf_Debug dbg,
-    Dwarf_CU_Context context,
-    Dwarf_Unsigned *abase_out,
-    Dwarf_Error *error)
-{
-    int res = 0;
-    Dwarf_Die cudie = 0;
-    Dwarf_Bool cu_die_offset_present = 0;
-    Dwarf_Unsigned cu_die_offset = 0;
-    Dwarf_Attribute myattr = 0;
-    if(context->cc_addr_base_present) {
-        *abase_out = context->cc_addr_base;
-        return DW_DLV_OK;
-    }
-
-    cu_die_offset = context->cc_cu_die_global_sec_offset;
-    cu_die_offset_present = context->cc_cu_die_offset_present;
-    if(!cu_die_offset_present) {
-        _dwarf_error(dbg, error,
-            DW_DLE_DEBUG_CU_UNAVAILABLE_FOR_FORM);
-        return (DW_DLV_ERROR);
-
-    }
-    res = dwarf_offdie_b(dbg,cu_die_offset,
-        context->cc_is_info,
-        &cudie,
-        error);
-    if(res != DW_DLV_OK) {
-        return res;
-    }
-    res = dwarf_attr(cudie,DW_AT_addr_base,
-        &myattr,error);
-    if(res == DW_DLV_ERROR) {
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        return res;
-    }
-    if (res == DW_DLV_OK) {
-        Dwarf_Unsigned val = 0;
-        res = dwarf_formudata(myattr,&val,error);
-        dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        if(res != DW_DLV_OK) {
-            return res;
-        }
-        *abase_out  = val;
-        return DW_DLV_OK;
-    }
-    /* NO ENTRY, try the other attr. */
-    res = dwarf_attr(cudie,DW_AT_GNU_addr_base, &myattr,error);
-    if(res == DW_DLV_NO_ENTRY) {
-        res = dwarf_attr(cudie,DW_AT_addr_base, &myattr,error);
-        if (res == DW_DLV_NO_ENTRY) {
-            /*  A .o or .dwp needs a base, but a .dwo does not.
-                FIXME: check this claim...
-                Assume zero is ok and works. */
-            *abase_out = 0;
-            dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-            return DW_DLV_OK;
-        }
-        if (res == DW_DLV_ERROR) {
-            dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-            return res;
-        }
-    } else if (res == DW_DLV_ERROR) {
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        return res;
-    }
-
-    {
-        Dwarf_Unsigned val = 0;
-        res = dwarf_formudata(myattr,&val,error);
-        dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        if(res != DW_DLV_OK) {
-            return res;
-        }
-        *abase_out  = val;
-    }
-    return DW_DLV_OK;
-}
 
 
-/* The dbg here will be the tieddbg, and context will be
-   a tied context.  */
-static int
-_dwarf_get_ranges_base_attr_value(Dwarf_Debug dbg,
-    Dwarf_CU_Context context,
-    Dwarf_Unsigned * rangesbase_out,
-    Dwarf_Error    * error)
-{
-    int res = 0;
-    Dwarf_Die cudie = 0;
-    Dwarf_Bool cu_die_offset_present = 0;
-    Dwarf_Unsigned cu_die_offset = 0;
-    Dwarf_Attribute myattr = 0;
-    if(context->cc_ranges_base_present) {
-        *rangesbase_out = context->cc_ranges_base;
-        return DW_DLV_OK;
-    }
-
-    cu_die_offset = context->cc_cu_die_global_sec_offset;
-    cu_die_offset_present = context->cc_cu_die_offset_present;
-    if(!cu_die_offset_present) {
-        _dwarf_error(dbg, error,
-            DW_DLE_DEBUG_CU_UNAVAILABLE_FOR_FORM);
-        return (DW_DLV_ERROR);
-
-    }
-    res = dwarf_offdie_b(dbg,cu_die_offset,
-        context->cc_is_info,
-        &cudie,
-        error);
-    if(res != DW_DLV_OK) {
-        return res;
-    }
-    res = dwarf_attr(cudie,DW_AT_rnglists_base,
-        &myattr,error);
-    if(res == DW_DLV_ERROR) {
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        return res;
-    }
-    if (res == DW_DLV_OK) {
-        Dwarf_Unsigned val = 0;
-        res = dwarf_formudata(myattr,&val,error);
-        dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        if(res != DW_DLV_OK) {
-            return res;
-        }
-        *rangesbase_out  = val;
-        return DW_DLV_OK;
-    }
-    /* NO ENTRY, try the other attr. */
-    res = dwarf_attr(cudie,DW_AT_GNU_ranges_base, &myattr,error);
-    if(res == DW_DLV_NO_ENTRY) {
-        res = dwarf_attr(cudie,DW_AT_rnglists_base, &myattr,error);
-        if (res == DW_DLV_NO_ENTRY) {
-            /*  A .o or execeutable skeleton  needs
-                a base , but a .dwo does not.
-                Assume zero is ok and works. */
-            *rangesbase_out = 0;
-            dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-            return DW_DLV_OK;
-        }
-        if (res == DW_DLV_ERROR) {
-            dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-            return res;
-        }
-    } else if (res == DW_DLV_ERROR) {
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        return res;
-    }
-
-    {
-        Dwarf_Unsigned val = 0;
-        res = dwarf_formudata(myattr,&val,error);
-        dwarf_dealloc(dbg,myattr,DW_DLA_ATTR);
-        dwarf_dealloc(dbg,cudie,DW_DLA_DIE);
-        if(res != DW_DLV_OK) {
-            return res;
-        }
-        *rangesbase_out  = val;
-    }
-    return DW_DLV_OK;
-}
 /*  This works for  all versions of DWARF.
     This is the preferred interface, cease using dwarf_highpc.
     The consumer has to check the return_form or
@@ -1285,8 +1291,7 @@ dwarf_highpc_b(Dwarf_Die die,
 
     if (class == DW_FORM_CLASS_ADDRESS) {
         Dwarf_Addr addr = 0;
-        if (attr_form == DW_FORM_GNU_addr_index ||
-            attr_form == DW_FORM_addrx) {
+        if (dwarf_addr_form_is_indexed(attr_form)) {
             Dwarf_Unsigned addr_out = 0;
             Dwarf_Unsigned index_to_addr = 0;
             int res2 = 0;
@@ -1300,48 +1305,15 @@ dwarf_highpc_b(Dwarf_Die die,
             if(res2 != DW_DLV_OK) {
                 return res2;
             }
-
-            res2 = _dwarf_extract_address_from_debug_addr(dbg,
+            res2= _dwarf_look_in_local_and_tied_by_index(dbg,
                 context,
                 index_to_addr,
                 &addr_out,
                 error);
             if(res2 != DW_DLV_OK) {
-                if (res2 == DW_DLV_ERROR &&
-                    error &&
-                    dwarf_errno(*error) ==
-                    DW_DLE_MISSING_NEEDED_DEBUG_ADDR_SECTION
-                    && dbg->de_tied_data.td_tied_object) {
-                    /*  .debug_addr is in tied dbg. */
-                    int res3 = 0;
-
-                    /*  Do not leak the above error pointer,
-                        we have something else to try here. */
-                    dwarf_dealloc(dbg,*error, DW_DLA_ERROR);
-                    *error = 0;
-
-                    /*  .debug_addr is in tied dbg.
-                        Get the index of the addr */
-                    res3 = _dwarf_get_addr_from_tied(dbg,
-                        context,index_to_addr,&addr_out,error);
-                    if ( res3 != DW_DLV_OK) {
-                        return res3;
-                    }
-                } else {
-                    return res2;
-                }
+                return res2;
             }
-            *return_value = addr_out;
-            /*  Allow null args starting 22 April 2019. */
-            if (return_form) {
-                *return_form = attr_form;
-            }
-            if (return_class) {
-                *return_class = class;
-            }
-            return (DW_DLV_OK);
         }
-
         READ_UNALIGNED_CK(dbg, addr, Dwarf_Addr,
             info_ptr, address_size,
             error,die_info_end);
@@ -1416,9 +1388,9 @@ _dwarf_get_addr_from_tied(Dwarf_Debug dbg,
         _dwarf_error(dbg, error, DW_DLE_NO_TIED_ADDR_AVAILABLE);
         return  DW_DLV_ERROR;
     }
-    if (!context->cc_signature_present) {
-        _dwarf_error(dbg, error, DW_DLE_NO_TIED_SIG_AVAILABLE);
-        return  DW_DLV_ERROR;
+    if(! context->cc_addr_base_present) {
+        /*  Does not exist. */
+        return DW_DLV_NO_ENTRY;
     }
     res = _dwarf_search_for_signature(tieddbg,
         context->cc_signature,
@@ -1448,55 +1420,6 @@ _dwarf_get_addr_from_tied(Dwarf_Debug dbg,
     return DW_DLV_OK;
 }
 
-int
-_dwarf_get_ranges_base_attr_from_tied(Dwarf_Debug dbg,
-    Dwarf_CU_Context context,
-    Dwarf_Unsigned * ranges_base_out,
-    Dwarf_Unsigned * addr_base_out,
-    Dwarf_Error*error)
-{
-    Dwarf_Debug tieddbg = 0;
-    int res = 0;
-    Dwarf_Unsigned tiedbase= 0;
-    Dwarf_CU_Context tiedcontext = 0;
-
-    if (!context->cc_signature_present) {
-        _dwarf_error(dbg, error, DW_DLE_NO_SIGNATURE_TO_LOOKUP);
-        return  DW_DLV_ERROR;
-    }
-    tieddbg = dbg->de_tied_data.td_tied_object;
-    if (!tieddbg) {
-        _dwarf_error(dbg, error, DW_DLE_NO_TIED_ADDR_AVAILABLE);
-        return  DW_DLV_ERROR;
-    }
-    if (!context->cc_signature_present) {
-        _dwarf_error(dbg, error, DW_DLE_NO_TIED_SIG_AVAILABLE);
-        return  DW_DLV_ERROR;
-    }
-    res = _dwarf_search_for_signature(tieddbg,
-        context->cc_signature,
-        &tiedcontext,
-        error);
-    if ( res == DW_DLV_ERROR) {
-        /* Associate the error with dbg, not tidedbg */
-        _dwarf_error_mv_s_to_t(tieddbg,error,dbg,error);
-        return res;
-    } else if ( res != DW_DLV_NO_ENTRY) {
-        return res;
-    }
-    res = _dwarf_get_ranges_base_attr_value(tieddbg, tiedcontext,
-        &tiedbase, error);
-    if (res != DW_DLV_OK) {
-        /* Associate the error with dbg, not tidedbg */
-        _dwarf_error_mv_s_to_t(tieddbg,error,dbg,error);
-        return res;
-    }
-    *ranges_base_out = tiedbase;
-    *addr_base_out =  tiedcontext->cc_addr_base;
-    return DW_DLV_OK;
-}
-
-
 /*
     Takes a die, an attribute attr, and checks if attr
     occurs in die.  Attr is required to be an attribute
@@ -1670,7 +1593,9 @@ dwarf_die_abbrev_code(Dwarf_Die die)
 }
 
 /*  Returns a flag through ablhas_child. Non-zero if
-    the DIE has children, zero if it does not.   */
+    the DIE has children, zero if it does not.
+    It has no Dwarf_Error arg!
+*/
 int
 dwarf_die_abbrev_children_flag(Dwarf_Die die,Dwarf_Half *ab_has_child)
 {
@@ -1719,9 +1644,9 @@ dw_get_special_offset(Dwarf_Half attrnum,
     case DW_AT_use_location:
     case DW_AT_vtable_elem_location: {
         if (dwversion <= 4) {
-            return DW_FORM_CLASS_LOCLISTPTR;
+            return DW_FORM_CLASS_LOCLIST;
         }
-        return DW_FORM_CLASS_LOCLIST;
+        return DW_FORM_CLASS_LOCLISTPTR;
         }
     case DW_AT_sibling:
     case DW_AT_byte_size :
