@@ -1,27 +1,31 @@
 /*
   Copyright (C) 2000-2005 Silicon Graphics, Inc.  All Rights Reserved.
-  Portions Copyright (C) 2007-2019 David Anderson. All Rights Reserved.
+  Portions Copyright (C) 2007-2020 David Anderson. All Rights Reserved.
   Portions Copyright 2012 SN Systems Ltd. All rights reserved.
+  Portions Copyright 2020 Google All rights reserved.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of version 2.1 of the GNU Lesser General Public License
-  as published by the Free Software Foundation.
+  This program is free software; you can redistribute it
+  and/or modify it under the terms of version 2.1 of the
+  GNU Lesser General Public License as published by the Free
+  Software Foundation.
 
-  This program is distributed in the hope that it would be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  This program is distributed in the hope that it would be
+  useful, but WITHOUT ANY WARRANTY; without even the implied
+  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+  PURPOSE.
 
-  Further, this software is distributed without any warranty that it is
-  free of the rightful claim of any third person regarding infringement
-  or the like.  Any license provided herein, whether implied or
-  otherwise, applies only to this software file.  Patent licenses, if
-  any, provided herein do not apply to combinations of this program with
-  other software, or any other product whatsoever.
+  Further, this software is distributed without any warranty
+  that it is free of the rightful claim of any third person
+  regarding infringement or the like.  Any license provided
+  herein, whether implied or otherwise, applies only to this
+  software file.  Patent licenses, if any, provided herein
+  do not apply to combinations of this program with other
+  software, or any other product whatsoever.
 
-  You should have received a copy of the GNU Lesser General Public
-  License along with this program; if not, write the Free Software
-  Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston MA 02110-1301,
-  USA.
+  You should have received a copy of the GNU Lesser General
+  Public License along with this program; if not, write the
+  Free Software Foundation, Inc., 51 Franklin Street - Fifth
+  Floor, Boston MA 02110-1301, USA.
 
 */
 
@@ -40,8 +44,12 @@
 #elif defined(_WIN32) && defined(_MSC_VER)
 #include <io.h>
 #endif /* HAVE_UNISTD_H */
-#include <sys/types.h> /* for open() */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h> /* open(), off_t, size_t, ssize_t */
+#endif /* HAVE_SYS_TYPES_H */
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h> /* for open() */
+#endif /*  HAVE_SYS_STAT_H */
 #include <fcntl.h> /* for open() */
 #include "dwarf_incl.h"
 #include "dwarf_alloc.h"
@@ -50,6 +58,7 @@
 #include "dwarf_abbrev.h"
 #include "memcpy_swap.h"
 #include "dwarf_die_deliv.h"
+#include "dwarfstring.h"
 #include "pro_encode_nm.h"
 
 #ifndef O_BINARY
@@ -141,6 +150,28 @@ _dwarf_file_has_debug_fission_index(Dwarf_Debug dbg)
     return FALSE;
 }
 
+void
+_dwarf_create_area_len_error(Dwarf_Debug dbg, Dwarf_Error *error,
+    Dwarf_Unsigned targ,
+    Dwarf_Unsigned sectionlen)
+{
+    dwarfstring m;
+
+    dwarfstring_constructor(&m);
+    dwarfstring_append_printf_u(&m,
+        "DW_DLE_HEADER_LEN_BIGGER_THAN_SECSIZE: "
+        " The header length of 0x%x is larger",
+        targ);
+    dwarfstring_append_printf_u(&m," than the "
+        "section length of 0x%x.",sectionlen);
+    _dwarf_error_string(dbg,
+        error,DW_DLE_HEADER_LEN_BIGGER_THAN_SECSIZE,
+        dwarfstring_string(&m));
+    dwarfstring_destructor(&m);
+}
+
+
+
 int
 _dwarf_internal_get_die_comp_dir(Dwarf_Die die, const char **compdir_out,
     const char **compname_out,
@@ -270,17 +301,26 @@ _dwarf_get_size_of_val(Dwarf_Debug dbg,
         return DW_DLV_OK;
 
     case DW_FORM_block1: {
-        ptrdiff_t sizeasptrdiff = 0;
+        Dwarf_Unsigned space_left = 0;
 
         if (val_ptr >= section_end_ptr) {
-            _dwarf_error(dbg,error,DW_DLE_FORM_BLOCK_LENGTH_ERROR);
+            _dwarf_error_string(dbg,error,
+                DW_DLE_FORM_BLOCK_LENGTH_ERROR,
+                "DW_DLE_FORM_BLOCK_LENGTH_ERROR: DW_FORM_block1"
+                " itself is off the end of the section."
+                " Corrupt Dwarf.");
             return DW_DLV_ERROR;
         }
         ret_value =  *(Dwarf_Small *) val_ptr;
-        sizeasptrdiff = (ptrdiff_t)ret_value;
-        if (sizeasptrdiff > (section_end_ptr - val_ptr) ||
-            sizeasptrdiff < 0) {
-            _dwarf_error(dbg,error,DW_DLE_FORM_BLOCK_LENGTH_ERROR);
+        /*  ptrdiff_t is generated but not named */
+        space_left = (section_end_ptr >= val_ptr)?
+            (section_end_ptr - val_ptr):0;
+        if (ret_value > space_left)  {
+            _dwarf_error_string(dbg,error,
+                DW_DLE_FORM_BLOCK_LENGTH_ERROR,
+                "DW_DLE_FORM_BLOCK_LENGTH_ERROR: DW_FORM_block1"
+                " runs off the end of the section."
+                " Corrupt Dwarf.");
             return DW_DLV_ERROR;
         }
         *size_out = ret_value +1;
@@ -288,14 +328,19 @@ _dwarf_get_size_of_val(Dwarf_Debug dbg,
         return DW_DLV_OK;
 
     case DW_FORM_block2: {
-        ptrdiff_t sizeasptrdiff = 0;
+        Dwarf_Unsigned space_left = 0;
 
         READ_UNALIGNED_CK(dbg, ret_value, Dwarf_Unsigned,
             val_ptr, DWARF_HALF_SIZE,error,section_end_ptr);
-        sizeasptrdiff = (ptrdiff_t)ret_value;
-        if (sizeasptrdiff > (section_end_ptr - val_ptr) ||
-            sizeasptrdiff < 0) {
-            _dwarf_error(dbg,error,DW_DLE_FORM_BLOCK_LENGTH_ERROR);
+        /*  ptrdiff_t is generated but not named */
+        space_left = (section_end_ptr >= val_ptr)?
+            (section_end_ptr - val_ptr):0;
+        if (ret_value > space_left)  {
+            _dwarf_error_string(dbg,error,
+                DW_DLE_FORM_BLOCK_LENGTH_ERROR,
+                "DW_DLE_FORM_BLOCK_LENGTH_ERROR: DW_FORM_block2"
+                " runs off the end of the section."
+                " Corrupt Dwarf.");
             return DW_DLV_ERROR;
         }
         *size_out = ret_value + DWARF_HALF_SIZE;
@@ -303,15 +348,20 @@ _dwarf_get_size_of_val(Dwarf_Debug dbg,
         return DW_DLV_OK;
 
     case DW_FORM_block4: {
-        ptrdiff_t sizeasptrdiff = 0;
+        Dwarf_Unsigned space_left = 0;
 
         READ_UNALIGNED_CK(dbg, ret_value, Dwarf_Unsigned,
             val_ptr, DWARF_32BIT_SIZE,
             error,section_end_ptr);
-        sizeasptrdiff = (ptrdiff_t)ret_value;
-        if (sizeasptrdiff > (section_end_ptr - val_ptr) ||
-            sizeasptrdiff < 0) {
-            _dwarf_error(dbg,error,DW_DLE_FORM_BLOCK_LENGTH_ERROR);
+        /*  ptrdiff_t is generated but not named */
+        space_left = (section_end_ptr >= val_ptr)?
+            (section_end_ptr - val_ptr):0;
+        if (ret_value > space_left)  {
+            _dwarf_error_string(dbg,error,
+                DW_DLE_FORM_BLOCK_LENGTH_ERROR,
+                "DW_DLE_FORM_BLOCK_LENGTH_ERROR: DW_FORM_block4"
+                " runs off the end of the section."
+                " Corrupt Dwarf.");
             return DW_DLV_ERROR;
         }
         *size_out = ret_value + DWARF_32BIT_SIZE;
@@ -459,7 +509,6 @@ _dwarf_get_size_of_val(Dwarf_Debug dbg,
     case DW_FORM_ref_sup8:
         *size_out = 8;
         return DW_DLV_OK;
-
     case DW_FORM_addrx1:
         *size_out = 1;
         return DW_DLV_OK;
@@ -580,6 +629,26 @@ _dwarf_valid_form_we_know(Dwarf_Unsigned at_form,
     return FALSE;
 }
 
+int
+_dwarf_format_TAG_err_msg(Dwarf_Debug dbg,
+    Dwarf_Unsigned tag,
+    const char *m,
+    Dwarf_Error *errp)
+{
+    dwarfstring v;
+
+    dwarfstring_constructor(&v);
+    dwarfstring_append(&v,(char *)m);
+    dwarfstring_append(&v," The value ");
+    dwarfstring_append_printf_u(&v,"0x%" DW_PR_DUx
+        " is outside the valid TAG range.",tag);
+    dwarfstring_append(&v," Corrupt DWARF.");
+    _dwarf_error_string(dbg, errp,DW_DLE_TAG_CORRUPT,
+        dwarfstring_string(&v));
+    dwarfstring_destructor(&v);
+    return DW_DLV_ERROR;
+}
+
 /*  This function returns a pointer to a Dwarf_Abbrev_List_s
     struct for the abbrev with the given code.  It puts the
     struct on the appropriate hash table.  It also adds all
@@ -606,15 +675,22 @@ _dwarf_valid_form_we_know(Dwarf_Unsigned at_form,
 
     See also dwarf_get_abbrev() in dwarf_abbrev.c.
 
+    On returning DW_DLV_NO_ENTRY (as well
+    as DW_DLV_OK)  it sets
+    *highest_known_code as a side effect
+    for better error messages by callers.
+
     Returns DW_DLV_ERROR on error.  */
 int
 _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
     Dwarf_Unsigned code,
     Dwarf_Abbrev_List *list_out,
+    Dwarf_Unsigned    *highest_known_code,
     Dwarf_Error *error)
 {
     Dwarf_Debug dbg = cu_context->cc_dbg;
-    Dwarf_Hash_Table hash_table_base = cu_context->cc_abbrev_hash_table;
+    Dwarf_Hash_Table hash_table_base =
+        cu_context->cc_abbrev_hash_table;
     Dwarf_Hash_Table_Entry entry_base = 0;
     Dwarf_Hash_Table_Entry entry_cur = 0;
     Dwarf_Unsigned hash_num = 0;
@@ -636,21 +712,26 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
             DW_DLA_HASH_TABLE_ENTRY,
             hash_table_base->tb_table_entry_count);
         if (!hash_table_base->tb_entries) {
+            *highest_known_code =
+                cu_context->cc_highest_known_code;
             return DW_DLV_NO_ENTRY;
         }
-
     } else if (hash_table_base->tb_total_abbrev_count >
-        ( hash_table_base->tb_table_entry_count * HT_MULTIPLE) ) {
+        (hash_table_base->tb_table_entry_count * HT_MULTIPLE)) {
         struct Dwarf_Hash_Table_s newht;
+
+        memset(&newht,0,sizeof(newht));
         /* Effectively multiplies by >= HT_MULTIPLE */
-        newht.tb_table_entry_count =  hash_table_base->tb_total_abbrev_count;
+        newht.tb_table_entry_count =
+            hash_table_base->tb_total_abbrev_count;
         newht.tb_total_abbrev_count = 0;
         newht.tb_entries =
-            (struct  Dwarf_Hash_Table_Entry_s *)_dwarf_get_alloc(dbg,
-            DW_DLA_HASH_TABLE_ENTRY,
+            (struct  Dwarf_Hash_Table_Entry_s *)
+            _dwarf_get_alloc(dbg, DW_DLA_HASH_TABLE_ENTRY,
             newht.tb_table_entry_count);
-
         if (!newht.tb_entries) {
+            *highest_known_code =
+                cu_context->cc_highest_known_code;
             return DW_DLV_NO_ENTRY;
         }
         /*  Copy the existing entries to the new table,
@@ -658,13 +739,17 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
         copy_abbrev_table_to_new_table(hash_table_base, &newht);
         /*  Dealloc only the entries hash table array, not the lists
             of things pointed to by a hash table entry array. */
-        dwarf_dealloc(dbg, hash_table_base->tb_entries,DW_DLA_HASH_TABLE_ENTRY);
+        dwarf_dealloc(dbg, hash_table_base->tb_entries,
+            DW_DLA_HASH_TABLE_ENTRY);
         hash_table_base->tb_entries = 0;
         /*  Now overwrite the existing table descriptor with
             the new, newly valid, contents. */
         *hash_table_base = newht;
     } /* Else is ok as is, add entry */
 
+    if (code > cu_context->cc_highest_known_code) {
+        cu_context->cc_highest_known_code = code;
+    }
     hashable_val = code;
     hash_num = hashable_val %
         hash_table_base->tb_table_entry_count;
@@ -673,11 +758,14 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
 
     /* Determine if the 'code' is the list of synonyms already. */
     for (hash_abbrev_entry = entry_cur->at_head;
-        hash_abbrev_entry != NULL && hash_abbrev_entry->abl_code != code;
+        hash_abbrev_entry != NULL &&
+        hash_abbrev_entry->abl_code != code;
         hash_abbrev_entry = hash_abbrev_entry->abl_next);
-    if (hash_abbrev_entry != NULL) {
-        /*  This returns a pointer to an abbrev list entry, not
-            the list itself. */
+    if (hash_abbrev_entry) {
+        /*  This returns a pointer to an abbrev
+            list entry, not the list itself. */
+        *highest_known_code =
+            cu_context->cc_highest_known_code;
         *list_out = hash_abbrev_entry;
         return DW_DLV_OK;
     }
@@ -713,15 +801,20 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
         This can happen,though it seems wrong.
         Or we are at the end of the data block,
         which we also take as
-        meaning done with abbrevs for this CU. An abbreviations table
-        is supposed to end with a zero byte. Not ended by end
-        of data block.  But we are allowing what is possibly a bit
+        meaning done with abbrevs for this CU.
+        An abbreviations table
+        is supposed to end with a zero byte.
+        Not ended by end of data block.
+        But we are allowing what is possibly a bit
         more flexible end policy here. */
     if (abbrev_ptr >= end_abbrev_ptr) {
         return DW_DLV_NO_ENTRY;
     }
-    /*  End of abbrev's for this cu, since abbrev code is 0. */
+    /*  End of abbrev's for this cu, since abbrev code
+        is 0. */
     if (*abbrev_ptr == 0) {
+        *highest_known_code =
+            cu_context->cc_highest_known_code;
         return DW_DLV_NO_ENTRY;
     }
 
@@ -738,23 +831,26 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
         DECODE_LEB128_UWORD_CK(abbrev_ptr, abbrev_tag,
             dbg,error,end_abbrev_ptr);
         if (abbrev_tag > DW_TAG_hi_user) {
-            _dwarf_error(dbg, error,DW_DLE_TAG_CORRUPT);
-            return DW_DLV_ERROR;
+            return _dwarf_format_TAG_err_msg(dbg,
+                abbrev_tag,"DW_DLE_TAG_CORRUPT",
+                error);
         }
-
         if (abbrev_ptr >= end_abbrev_ptr) {
             _dwarf_error(dbg, error, DW_DLE_ABBREV_OFF_END);
             return DW_DLV_ERROR;
         }
-
         inner_list_entry = (Dwarf_Abbrev_List)
-            _dwarf_get_alloc(cu_context->cc_dbg, DW_DLA_ABBREV_LIST, 1);
+            _dwarf_get_alloc(cu_context->cc_dbg,
+                DW_DLA_ABBREV_LIST, 1);
         if (inner_list_entry == NULL) {
             _dwarf_error(dbg, error, DW_DLE_ALLOC_FAIL);
             return DW_DLV_ERROR;
         }
 
         new_hashable_val = abbrev_code;
+        if (abbrev_code > cu_context->cc_highest_known_code) {
+            cu_context->cc_highest_known_code = abbrev_code;
+        }
         hash_num = new_hashable_val %
             hash_table_base->tb_table_entry_count;
         inner_hash_entry = entry_base + hash_num;
@@ -765,17 +861,21 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
         hash_table_base->tb_total_abbrev_count++;
 
         inner_list_entry->abl_code = abbrev_code;
+
         inner_list_entry->abl_tag = abbrev_tag;
         inner_list_entry->abl_has_child = *(abbrev_ptr++);
         inner_list_entry->abl_abbrev_ptr = abbrev_ptr;
         inner_list_entry->abl_goffset =  abb_goff;
         hash_table_base->tb_total_abbrev_count++;
 
-        /*  Cycle thru the abbrev content, ignoring the content except
+        /*  Cycle thru the abbrev content,
+            ignoring the content except
             to find the end of the content. */
         res = _dwarf_count_abbrev_entries(dbg,abbrev_ptr,
             end_abbrev_ptr,&atcount,&abbrev_ptr2,error);
         if (res != DW_DLV_OK) {
+            *highest_known_code =
+                cu_context->cc_highest_known_code;
             return res;
         }
         abbrev_ptr = abbrev_ptr2;
@@ -783,14 +883,16 @@ _dwarf_get_abbrev_for_code(Dwarf_CU_Context cu_context,
     } while ((abbrev_ptr < end_abbrev_ptr) &&
         *abbrev_ptr != 0 && abbrev_code != code);
 
+    *highest_known_code = cu_context->cc_highest_known_code;
     cu_context->cc_last_abbrev_ptr = abbrev_ptr;
     cu_context->cc_last_abbrev_endptr = end_abbrev_ptr;
     if(abbrev_code == code) {
         *list_out = inner_list_entry;
         return DW_DLV_OK;
     }
-    /*  We cannot find an abbrev_code  matching code. ERROR
-        will be declared eventually.  Might be better to declare
+    /*  We cannot find an abbrev_code  matching code.
+        ERROR will be declared eventually.
+        Might be better to declare
         specific errors here? */
     return DW_DLV_NO_ENTRY;
 }
@@ -962,6 +1064,7 @@ _dwarf_length_of_cu_header(Dwarf_Debug dbg,
             cuptr, sizeof(Dwarf_Ubyte),error,section_end_ptr);
         switch (unit_type) {
         case DW_UT_compile:
+        case DW_UT_partial:
             final_size = local_extension_size +
                 local_length_size  + /* Size of cu length field. */
                 DWARF_HALF_SIZE + /* Size of version stamp field. */
@@ -970,10 +1073,26 @@ _dwarf_length_of_cu_header(Dwarf_Debug dbg,
                 local_length_size ;  /* Size of abbrev offset field. */
             break;
         case DW_UT_type:
-        case DW_UT_partial:
+        case DW_UT_split_type:
+            final_size = local_extension_size +
+                local_length_size  + /* Size of type unit length field. */
+                DWARF_HALF_SIZE + /* Size of version stamp field. */
+                sizeof(Dwarf_Small)+ /* Size of unit type field. */
+                sizeof(Dwarf_Small)+ /* Size of address size field. */
+                local_length_size +  /* Size of abbrev offset field. */
+                sizeof(Dwarf_Sig8) + /* Size of type signature field. */
+                local_length_size; /* Size of type offset field. */
+            break;
         case DW_UT_skeleton:
         case DW_UT_split_compile:
-        case DW_UT_split_type:
+            final_size = local_extension_size +
+                local_length_size +   /* Size of unit length field. */
+                DWARF_HALF_SIZE +     /* Size of version stamp field. */
+                sizeof(Dwarf_Small) + /* Size of unit type field. */
+                sizeof(Dwarf_Small) + /* Size of address size field. */
+                local_length_size +   /* Size of abbrev offset field. */
+                sizeof(Dwarf_Sig8);   /* Size of dwo id field. */
+            break;
         default:
             _dwarf_error(dbg,error,DW_DLE_UNIT_TYPE_NOT_HANDLED);
             return DW_DLV_ERROR;
@@ -1040,7 +1159,21 @@ _dwarf_load_debug_info(Dwarf_Debug dbg, Dwarf_Error * error)
         return res;
     }
     res = _dwarf_load_section(dbg, &dbg->de_debug_info, error);
-    return res;
+    if (res != DW_DLV_OK) {
+        return res;
+    }
+    /*  debug_info won't be meaningful without
+        .debug_rnglists and .debug_rnglists if there
+        is one or both such sections. */
+    res = dwarf_load_rnglists(dbg,0,error);
+    if (res == DW_DLV_ERROR) {
+        return res;
+    }
+    res = dwarf_load_loclists(dbg,0,error);
+    if (res == DW_DLV_ERROR) {
+        return res;
+    }
+    return DW_DLV_OK;
 }
 int
 _dwarf_load_debug_types(Dwarf_Debug dbg, Dwarf_Error * error)
@@ -1057,15 +1190,25 @@ _dwarf_load_debug_types(Dwarf_Debug dbg, Dwarf_Error * error)
     return res;
 }
 void
-_dwarf_free_abbrev_hash_table_contents(Dwarf_Debug dbg,Dwarf_Hash_Table hash_table)
+_dwarf_free_abbrev_hash_table_contents(Dwarf_Debug dbg,
+    Dwarf_Hash_Table hash_table)
 {
     /*  A Hash Table is an array with tb_table_entry_count struct
         Dwarf_Hash_Table_s entries in the array. */
     unsigned hashnum = 0;
+    if(!hash_table) {
+        /*  Not fully set up yet. There is nothing to do. */
+        return;
+    }
+    if (!hash_table->tb_entries) {
+        /*  Not fully set up yet. There is nothing to do. */
+        return;
+    }
     for (; hashnum < hash_table->tb_table_entry_count; ++hashnum) {
         struct Dwarf_Abbrev_List_s *abbrev = 0;
         struct Dwarf_Abbrev_List_s *nextabbrev = 0;
-        struct  Dwarf_Hash_Table_Entry_s *tb =  &hash_table->tb_entries[hashnum];
+        struct  Dwarf_Hash_Table_Entry_s *tb =
+            &hash_table->tb_entries[hashnum];
 
         abbrev = tb->at_head;
         for (; abbrev; abbrev = nextabbrev) {
@@ -1158,116 +1301,23 @@ dwarf_register_printf_callback( Dwarf_Debug dbg,
 }
 
 
-/*  Allocate a bigger buffer if necessary.
-    Do not worry about previous content of the buffer.
-    Return 0 if we fail here.
-    Else return the requested len value. */
-static unsigned buffersetsize(Dwarf_Debug dbg,
-    struct  Dwarf_Printf_Callback_Info_s *bufdata,
-    int len)
-{
-    char *space = 0;
 
-    if (!dbg->de_printf_callback_null_device_handle) {
-        FILE *de = fopen(NULL_DEVICE_NAME,"w");
-        if(!de) {
-            return 0;
-        }
-        dbg->de_printf_callback_null_device_handle = de;
-    }
-    if (bufdata->dp_buffer_user_provided) {
-        return bufdata->dp_buffer_len;
-    }
-    /* Make big enough for a trailing NUL char. */
-    space = (char *)malloc(len+1);
-    if (!space) {
-        /* Out of space, we cannot do anything. */
-        return 0;
-    }
-    free(bufdata->dp_buffer);
-    bufdata->dp_buffer = space;
-    bufdata->dp_buffer_len = len;
-    return len;
-}
-
-/*  We are only using C90 facilities, not C99,
-    in libdwarf/dwarfdump. */
+/* No varargs required */
 int
-dwarf_printf(Dwarf_Debug dbg,
-    const char * format,
-    ...)
+_dwarf_printf(Dwarf_Debug dbg,
+    const char * data)
 {
-    va_list ap;
-    unsigned bff = 0;
+    int nlen = 0;
     struct Dwarf_Printf_Callback_Info_s *bufdata =
         &dbg->de_printf_callback;
-    FILE * null_device_handle = 0;
 
     dwarf_printf_callback_function_type func = bufdata->dp_fptr;
     if (!func) {
         return 0;
     }
-    null_device_handle =
-        (FILE *) dbg->de_printf_callback_null_device_handle;
-    if (!bufdata->dp_buffer || !null_device_handle) {
-        /*  Sets dbg device handle for later use if not
-            set already. */
-        bff = buffersetsize(dbg,bufdata,MINBUFLEN);
-        if (!bff) {
-            /*  Something is wrong. */
-            return 0;
-        }
-        if (!bufdata->dp_buffer) {
-            /*  Something is wrong. Possibly caller
-                set up callback wrong. */
-            return 0;
-        }
-    }
-
-    {
-        int plen = 0;
-        int nlen = 0;
-        null_device_handle =
-            (FILE *) dbg->de_printf_callback_null_device_handle;
-        if (!null_device_handle) {
-            /*  Something is wrong. */
-            return 0;
-        }
-        va_start(ap,format);
-        plen = vfprintf(null_device_handle,format,ap);
-        va_end(ap);
-
-        if (!bufdata->dp_buffer_user_provided) {
-            if (plen >= (int)bufdata->dp_buffer_len) {
-                bff = buffersetsize(dbg,bufdata,plen+2);
-                if (!bff) {
-                    /*  Something is wrong.  */
-                    return 0;
-                }
-            }
-        } else {
-            if (plen >= (int)bufdata->dp_buffer_len) {
-                /*  We are stuck! User did not
-                    give us space needed!  */
-                return 0;
-            }
-        }
-
-        va_start(ap,format);
-        nlen = vsprintf(bufdata->dp_buffer,
-            format,ap);
-        va_end(ap);
-        if ( nlen > plen) {
-            /* Impossible. Memory is corrupted now */
-            fprintf(stderr,"\nlibdwarf impossible sprintf error %s %d\n",
-                __FILE__,__LINE__);
-            exit(1);
-        }
-        func(bufdata->dp_user_pointer,bufdata->dp_buffer);
-        return nlen;
-    }
-    /* Not reached. */
-    return 0;
+    nlen =  strlen(data);
+    func(bufdata->dp_user_pointer,data);
+    return nlen;
 }
 
 /*  Often errs and errt point to the same Dwarf_Error,
@@ -1345,6 +1395,15 @@ _dwarf_what_section_are_we(Dwarf_Debug dbg,
     FINDSEC(&dbg->de_debug_loc,
         our_pointer, section_name_out,
         sec_start_ptr_out, sec_len_out, sec_end_ptr_out);
+    FINDSEC(&dbg->de_debug_loclists,
+        our_pointer, section_name_out,
+        sec_start_ptr_out, sec_len_out, sec_end_ptr_out);
+    FINDSEC(&dbg->de_debug_rnglists,
+        our_pointer, section_name_out,
+        sec_start_ptr_out, sec_len_out, sec_end_ptr_out);
+    FINDSEC(&dbg->de_debug_addr,
+        our_pointer, section_name_out,
+        sec_start_ptr_out, sec_len_out, sec_end_ptr_out);
     FINDSEC(&dbg->de_debug_line,
         our_pointer, section_name_out,
         sec_start_ptr_out, sec_len_out, sec_end_ptr_out);
@@ -1412,3 +1471,84 @@ int  dwarf_add_file_path(
     }
     return DW_DLV_OK;
 }
+
+/*  New late April 2020.
+    All the crucial macros will surely
+    need  to use wrapper code to ensure we do not leak
+    memory at certain points.  */
+int
+_dwarf_read_unaligned_ck_wrapper(Dwarf_Debug dbg,
+    Dwarf_Unsigned *out_value,
+    Dwarf_Small *readfrom,
+    int          readlength,
+    Dwarf_Small *end_arange,
+    Dwarf_Error *err)
+{
+    Dwarf_Unsigned val = 0;
+
+    READ_UNALIGNED_CK(dbg,val,Dwarf_Unsigned,
+        readfrom,readlength,err,end_arange);
+    *out_value = val;
+    return DW_DLV_OK;
+}
+
+int
+_dwarf_read_area_length_ck_wrapper(Dwarf_Debug dbg,
+    Dwarf_Unsigned *out_value,
+    Dwarf_Small **readfrom,
+    int    *  length_size_out,
+    int    *  exten_size_out,
+    Dwarf_Unsigned sectionlength,
+    Dwarf_Small *endsection,
+    Dwarf_Error *err)
+{
+    Dwarf_Small *ptr = *readfrom;
+    Dwarf_Unsigned val = 0;
+    int length_size = 0;
+    int exten_size = 0;
+
+    READ_AREA_LENGTH_CK(dbg,val,Dwarf_Unsigned,
+        ptr,length_size,exten_size,
+        err,
+        sectionlength,endsection);
+    *readfrom = ptr;
+    *out_value = val;
+    *length_size_out = length_size;
+    *exten_size_out = exten_size;
+    return DW_DLV_OK;
+}
+/*  New March 2020 */
+/*  We need to increment startptr for the caller
+    in these wrappers so the caller passes in
+    wrappers return either DW_DLV_OK or DW_DLV_ERROR.
+    Never DW_DLV_NO_ENTRY. */
+int
+_dwarf_leb128_uword_wrapper(Dwarf_Debug dbg,
+    Dwarf_Small ** startptr,
+    Dwarf_Small * endptr,
+    Dwarf_Unsigned *out_value,
+    Dwarf_Error * error)
+{
+    Dwarf_Unsigned utmp2 = 0;
+    Dwarf_Small * start = *startptr;
+    DECODE_LEB128_UWORD_CK(start, utmp2,
+        dbg,error,endptr);
+    *out_value = utmp2;
+    *startptr = start;
+    return DW_DLV_OK;
+}
+int
+_dwarf_leb128_sword_wrapper(Dwarf_Debug dbg,
+    Dwarf_Small ** startptr,
+    Dwarf_Small * endptr,
+    Dwarf_Signed *out_value,
+    Dwarf_Error * error)
+{
+    Dwarf_Small * start = *startptr;
+    Dwarf_Signed stmp2 = 0;
+    DECODE_LEB128_SWORD_CK(start, stmp2,
+        dbg,error,endptr);
+    *out_value = stmp2;
+    *startptr = start;
+    return DW_DLV_OK;
+}
