@@ -1,26 +1,29 @@
 /*
     Copyright 2009-2010 SN Systems Ltd. All rights reserved.
-    Portions Copyright 2009-2018 David Anderson. All rights reserved.
+    Portions Copyright 2009-2020 David Anderson. All rights reserved.
 
-    This program is free software; you can redistribute it and/or modify it
-    under the terms of version 2.1 of the GNU Lesser General Public License
-    as published by the Free Software Foundation.
+    This program is free software; you can redistribute it
+    and/or modify it under the terms of version 2.1 of the
+    GNU Lesser General Public License as published by the
+    Free Software Foundation.
 
-    This program is distributed in the hope that it would be useful, but
-    WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    This program is distributed in the hope that it would
+    be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A
+    PARTICULAR PURPOSE.
 
-    Further, this software is distributed without any warranty that it is
-    free of the rightful claim of any third person regarding infringement
-    or the like.  Any license provided herein, whether implied or
-    otherwise, applies only to this software file.  Patent licenses, if
-    any, provided herein do not apply to combinations of this program with
+    Further, this software is distributed without any warranty
+    that it is free of the rightful claim of any third person
+    regarding infringement or the like.  Any license provided
+    herein, whether implied or otherwise, applies only to
+    this software file.  Patent licenses, if any, provided
+    herein do not apply to combinations of this program with
     other software, or any other product whatsoever.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this program; if not, write the Free Software
-    Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston MA 02110-1301,
-    USA.
+    You should have received a copy of the GNU Lesser General
+    Public License along with this program; if not, write
+    the Free Software Foundation, Inc., 51 Franklin Street -
+    Fifth Floor, Boston MA 02110-1301, USA.
 
 */
 
@@ -35,7 +38,9 @@
 #endif /* HAVE_STDLIB_H */
 #include <errno.h>   /* For errno declaration. */
 #include <ctype.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif /* HAVE_STRING_H */
 #include "dwgetopt.h"
 #include "libdwarf_version.h" /* for DW_VERSION_DATE_STR */
 
